#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/4/11 13:35
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import os
import time
import warnings

from .commonCode import checkOS

__all__ = ['osprint', 'osprint2', 'cprintf']


def osprint(_str):
    """
    打印信息
    :param _str: 打印信息
    :return:
    """
    if not checkOS('WIN'):
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> {_str}"')
    else:
        ddt_str = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
        print(f'{ddt_str} ---> {_str}')


def osprint2(_status, _str):
    """
    打印信息
    :param _status: 状态
    :param _str: 打印信息
    :return:
    """
    if not checkOS('WIN'):
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> [{_status}] {_str}"')
    else:
        ddt_str = time.strftime('%Y-%m-%d %H:%M:%S', time.localtime())
        print(f'{ddt_str} ---> [{_status}] {_str}')


def cprintf(_status=None, _str=None, _voice=True, _log_level='INFO'):
    """
    以颜色打印信息
    :param _status: 状态
    :param _str: 打印信息
    :param _voice: 是否语音提示
    :param _log_level: 日志级别
    :return:
    """
    warnings.simplefilter('always', DeprecationWarning)
    if not _status:
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> {_str}"')
    elif _status == 'SUCCESS':  # 绿色
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> \033[1;32m[{_status}]\033[0m\033[32m {_str}\033[0m"')
    elif _status == 'INFO':  # 蓝色
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> \033[1;34m[{_status}]\033[0m\033[34m {_str}\033[0m"')
    elif _status == 'WARNING':  # 橙色
        os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> \033[1;33m[{_status}]\033[0m\033[33m {_str}\033[0m"')
    elif _status == 'ERROR':  # 红色
        if _voice:
            os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> \033[1;31m[{_status}]\033[0m\033[31m {_str}\033[0m\a"')
        else:
            os.system(f'echo -e `date "+%Y-%m-%d %T"` "---> \033[1;31m[{_status}]\033[0m\033[31m {_str}\033[0m"')



# -*- coding: utf-8 -*-

# @File    : interp_s2g.py
# @Date    : 2023-03-27
# @Author  : Dovelet
"""
程序算法：必须严格限制范围，拒绝遍历所有点！
先算水点的最近格点，再算格点的最近水点
Have done!!!
"""
import itertools
import json
import os
from collections import deque

import matplotlib.pyplot as plt
import numpy as np
import scipy.spatial as spt
from netCDF4 import Dataset

from ... import Blog

__all__ = ['s_search_g', 'save_id', 'read_id', 'get_interp', 'check_weight_exist', 'draw_value',
           'find_index', 'region_define', 'g_search_s']


logger_interps2g = Blog("interp_s2g").logger


def find_index(g_lon, g_lat, _s_lon, _s_lat):
    """
    获取经纬度索引
    :param g_lon: 一维格点经度
    :param g_lat: 一维格点纬度
    :param _s_lon: 散点经度
    :param _s_lat: 散点纬度
    :return: 纬度索引 经度索引
    """
    _s_lon = np.array(_s_lon)
    _s_lat = np.array(_s_lat)
    _latli = np.where(g_lat == _s_lat[:, None])[-1]
    _lonli = np.where(g_lon == _s_lon[:, None])[-1]

    return _latli.tolist(), _lonli.tolist()


def region_define(lonmin, lonmax, latmin, latmax, re_ra):
    """
    插值范围的确定
    :param lonmin: 经度最小值
    :param lonmax: 经度最大值
    :param latmin: 纬度最小值
    :param latmax: 纬度最大值
    :param re_ra: 分辨率的倒数
    :return: 一维的经度和一维的纬度
    """
    lat_num_grid = int(round(latmax - latmin, 2) * re_ra)
    lon_num_grid = int(round(lonmax - lonmin, 2) * re_ra)

    glon = np.linspace(lonmin, lonmax, lon_num_grid)
    glat = np.linspace(latmin, latmax, lat_num_grid)
    return glon, glat


def s_search_g(s_lon, s_lat, lonmin, lonmax, latmin, latmax, re_ra, k=8):
    """
    提取需要插值的格点索引
    :param s_lon: 散点经度
    :param s_lat: 散点纬度
    :param lonmin: 网格经度最小值
    :param lonmax: 网格经度最大值
    :param latmin: 网格纬度最小值
    :param latmax: 网格纬度最大值
    :param re_ra: 分辨率倒数
    :param k: 寻找点的个数
    :param k: 寻找点的个数
    :return:
    """
    g_lon, g_lat=region_define(lonmin, lonmax, latmin, latmax, re_ra)
    lat_number = g_lat.shape[0]
    lon_number = g_lon.shape[0]
    grid = []
    grid=deque(grid)
    for i, j in itertools.product(range(lon_number), range(lat_number)):
        x = [g_lon[i], g_lat[j]]
        grid.append(x)
    point = np.array(grid)
    fp_lon = []
    fp_lon=deque(fp_lon)
    fp_lat = []
    fp_lat=deque(fp_lat)
    Fp=[]
    Fp=deque(Fp)

    # 用于快速查找的KDTree类
    ckt = spt.cKDTree(point)  # 用C写的查找类，执行速度更快
    logger_interps2g.debug('正在判断哪些格点需要赋值')
    for i in range(s_lon.shape[0]):
        if (latmin-1/re_ra<=s_lat[i]<=latmax+1/re_ra) and (lonmin-re_ra<=s_lon[i]<=lonmax+re_ra):
            find_point = np.array([s_lon[i], s_lat[i]])
            distance, sequence = ckt.query(find_point, k)

            for item in sequence:
                if grid[item] not in Fp:
                    Fp.append(grid[item])
                    fp_lon.append(grid[item][0])
                    fp_lat.append(grid[item][1])

    logger_interps2g.debug(f'判断完毕,共有{len(Fp)}个格点需要赋值')
    ilat, ilon = find_index(g_lon, g_lat, fp_lon, fp_lat)
    # 现在已经找齐了需要插值的格点，我们现在需要知道，这些格点都由谁来赋值
    logger_interps2g.info('赋值索引查找中')
    fill_id=g_search_s(s_lon, s_lat, Fp, k=1)

    logger_interps2g.debug('赋值索引查找完毕')
    return ilat, ilon, fill_id


def g_search_s(s_lon, s_lat, Fp, k=1):
    """
    根据格点找散点
    :param s_lon:
    :param s_lat:
    :param Fp:
    :param k:
    :return:
    """
    s_grid = []
    s_grid=deque(s_grid)
    for i in range(s_lon.shape[0]):
        s_grid.append([s_lon[i], s_lat[i]])
    s_point = np.array(s_grid)
    ckt = spt.cKDTree(s_point)  # 用C写的查找类，执行速度更快
    fill_id=[]
    fill_id=deque(fill_id)
    for ip in Fp:
        find_point = np.array(ip)
        distance, sequence = ckt.query(find_point, k)

        fill_id.append(sequence)

    return fill_id


def save_id(latli, lonli, fill_id, out_path):
    """
    保存权重
    :param latli: 纬度索引
    :param lonli: 经度索引
    :param fill_id: 赋值索引
    :param out_path: 输出文件地址
    :return:
    """
    weight={'latli': list(latli), 'lonli': list(lonli), 'fill_id': list(fill_id)}
    with open(out_path, 'w', encoding='utf-8') as f:
        json.dump(weight, f)


def read_id(weight_json):
    """
    读取权重
    :param weight_json: 权重文件地址
    :return:
    """
    with open(weight_json, 'r', encoding='utf-8') as f:
        weight = json.load(f)

    return weight


def get_interp(lonmin, lonmax, latmin, latmax, re_ra, s_data, weight):
    """
    插值
    :param lonmin:
    :param lonmax:
    :param latmin:
    :param latmax:
    :param re_ra:
    :param s_data:
    :param weight:
    :return:
    """
    olon, olat = region_define(lonmin, lonmax, latmin, latmax, re_ra)
    lat_number = olat.shape[0]
    lon_number = olon.shape[0]
    # make new matrix
    # Eg: (24,30,40,80)
    #      24: time
    #      30: level
    #      40: lat
    #      80: lon
    size_new = list(s_data.shape)
    size_new[-1] = lat_number
    size_new.append(lon_number)
    _new = np.full(size_new, np.nan)

    latli = weight['latli']
    lonli = weight['lonli']
    fill_id = weight['fill_id']
    if np.ndim(s_data) == 1:
        _new[latli, lonli] = s_data[fill_id]
    else:
        s_data = np.reshape(s_data, (-1, s_data.shape[-1]))
        _new = np.reshape(_new, (-1, _new.shape[-2], _new.shape[-1]))

        for i in range(s_data.shape[0]):
            _new[i, latli, lonli] = s_data[i, fill_id]
        _new = np.reshape(_new, size_new)

    return _new


def check_weight_exist(weight_path):
    """
    检查权重文件是否存在
    :param weight_path:
    :return:
    """
    return bool(os.path.exists(weight_path))


def draw_value(value):
    """
    画个图康康
    :param value:
    :return:
    """
    plt.figure()
    plt.contourf(value, cmap='rainbow')
    plt.colorbar()
    # plt.savefig('test2.png')
    plt.show()


def example():
    """
    example
    """
    lonmin = 119.75
    lonmax = 119.95
    latmin = 35.55
    latmax = 35.65
    re_ra = 400
    a = Dataset('/data/Output_160/ww3_djk/20230409/ww3.20230409.nc')
    s_lon = a.variables['longitude'][:]
    s_lat = a.variables['latitude'][:]
    hs = a.variables['hs'][:, :]
    latli, lonli, fill = s_search_g(s_lon, s_lat, lonmin, lonmax, latmin, latmax, re_ra, k=4)
    save_id(latli, lonli, fill, 'weight.json')
    weight = read_id('weight.json')
    _new = get_interp(lonmin, lonmax, latmin, latmax, re_ra, hs, weight)
    draw_value(_new[0, :])


if __name__ == '__main__':
    example()



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/12/13 20:47
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
"""

from datetime import datetime, timedelta

import numpy as np
from matplotlib import tri
from netCDF4 import Dataset

# from easydict import EasyDict
from .. import P_para, Ttime
from .fvcom_tools import calc_xcyc, check_grid_type, f_calc_nbve

_all_ = ['f_load_time', 'f_load_grid']


def f_load_time(fin, fmethod='time'):
    """
    Load FVCOM time
    :param fin: file name
    :param fmethod: 'time' or 'Times'
    return: Ttimes
    """

    ncid = Dataset(fin, 'r')

    if fmethod == 'Times' and 'Times' in ncid.variables.keys():
        Times_fvcom = ncid.variables['Times'][:]
        TIME = np.apply_along_axis(lambda row: b''.join(row).decode(), 1, Times_fvcom.data).tolist()
        Times = [datetime.fromisoformat(TIME[i]) for i in range(len(TIME))]
    else:
        Itime = ncid.variables['Itime'][:]
        Itime2 = ncid.variables['Itime2'][:]
        time = Itime + Itime2 / 1000 / 3600 / 24  # --> day
        if 'format' in ncid.variables['Itime'].ncattrs() and 'MJD' in ncid.variables['Itime'].format:
            Times = [datetime(1858, 11, 17) + timedelta(days=x) for x in time]
        else:
            return None
    Ttimes = Ttime(Times=Times)
    return Ttimes


def f_load_grid(fin,
                Coordinate='geo',
                MaxLon=180,
                Global=False,
                Nodisp=False):
    """
    Load FVCOM grid class
    :param fin: grid file
    :param Coordinate: 'geo' or 'xy'
    :param MaxLon: 180 or 360
    :param Global: True or False
    :param Nodisp: True or False
    :return: fgrid
    """
    
    # fgrid = EasyDict()
    fgrid = P_para()

    if fin.endswith('.nc'):
        ncid = Dataset(fin, 'r')
        if Coordinate == 'geo':
            fgrid.x = ncid.variables['lon'][:].data
            fgrid.y = ncid.variables['lat'][:].data
            fgrid.nv = ncid.variables['nv'][:].data
        else:
            fgrid.x = ncid.variables['x'][:].data
            fgrid.y = ncid.variables['y'][:].data
            fgrid.nv = ncid.variables['nv'][:].data
        
        fgrid.MaxLon = MaxLon
        fgrid.LON = fgrid.x
        fgrid.LAT = fgrid.y
        fgrid.nvMin0 = fgrid.nv - 1 if np.min(fgrid.nv) == 1 else fgrid.nv
        fgrid.nvMin1 = fgrid.nvMin0 + 1
        fgrid.tri = tri.Triangulation(fgrid.x, fgrid.y, fgrid.nvMin1.transpose()-1)
        
        fgrid.h = ncid.variables['h'][:] if 'h' in ncid.variables.keys() else None
        fgrid.siglay = ncid.variables['siglay'][:] if 'siglay' in ncid.variables.keys() else None
        ncid.close()
    
    elif fin.endswith('grd.dat'):  # 读取grd文件
        pass
    elif fin.endswith('.2dm'):  # 读取2dm文件
        pass
    elif fin.endswith('.msh'):  # 读取msh文件
        pass
    elif fin.endswith('.14'):  # 读取fort.14文件
        pass
    elif fin.endswith('.mesh'):  # 读取mike mesh文件
        pass
    else:
        raise ValueError('Unknown grid file format !!!')
    
    if Global:
        fgrid.type = 'Global'
    else:
        fgrid.type = check_grid_type(fgrid.x, fgrid.y)
    
    fgrid.xc, fgrid.yc = calc_xcyc(fgrid.x, fgrid.y, fgrid.nv, Global=[True if fgrid.type == 'Global' else False][0])
    fgrid.node = len(fgrid.x)
    fgrid.nele = len(fgrid.xc)
    fgrid.nbve = f_calc_nbve(fgrid)
    
    if 'siglay' in fgrid.keys():
        fgrid.kbm1 = fgrid.siglay.shape[0]
        fgrid.kb = fgrid.kbm1 + 1
    
    if not Nodisp:
        print(' ')
        print('------------------------------------------------')
        print('FVCOM grid:')
        print('   Dimension :  ')
        print(f'              node    : {fgrid.node}')
        print(f'              nele    : {fgrid.nele}')
        print(f'              nsiglay : {fgrid.kbm1}') if 'siglay' in fgrid.keys() else None
        print(f'   X / Longitude : {min(fgrid.x):.2f} ~ {max(fgrid.x):.2f}')
        print(f'   Y / Latitude  : {min(fgrid.y):.2f} ~ {max(fgrid.y):.2f}')
        print(f'   Depth         : {min(fgrid.h):.2f} ~ {max(fgrid.h):.2f}') if 'h' in fgrid.keys() else None
        print('------------------------------------------------')
        print(' ')
    
    return fgrid
    


#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/11/27 16:11
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
Converting Degrees, Minutes, Seconds formatted coordinate strings to decimal.

Formula:
DEC = (DEG + (MIN * 1/60) + (SEC * 1/60 * 1/60))

Assumes S/W are negative.
"""

import re

__all__ = ['dms2degree']


def dms2degree(dms_str):
    """
    Return decimal representation of DMS

    \\>>>dms2degree(utf8(48°53'10.18"N))
    48.8866111111F

    \\>>>dms2degree(utf8(2°20'35.09"E))
    2.34330555556F

    \\>>>dms2degree(utf8(48°53'10.18"S))
    -48.8866111111F

    \\>>>dms2degree(utf8(2°20'35.09"W))
    -2.34330555556F

    """
    
    dms_str = re.sub(r'\s', '', dms_str)

    if re.search('[NSEWnsew]', dms_str) is None:
        sign = -1 if re.search('[swSW]', dms_str) else 1
    else:
        sign = 1

    numbers = [*filter(len, re.split(r'\D+', dms_str, maxsplit=4))]

    degree = numbers[0]
    minute = numbers[1] if len(numbers) >= 2 else '0'
    second = numbers[2] if len(numbers) >= 3 else '0'
    frac_seconds = numbers[3] if len(numbers) >= 4 else '0'

    second += f".{frac_seconds}"
    return sign * (int(degree) + float(minute) / 60 + float(second) / 3600)



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2025/1/9 11:32
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
 
"""
import matplotlib.pyplot as plt
import numpy as np
from matplotlib import tri

_all_ = ['f_2d_mesh', 'f_2d_image']


def f_2d_mesh(fgrid, Global=False, Color='k'):
    """
    draw 2d mesh
    :param fgrid: FVCOM grid
    :param Global: True or False
    :param Color: color
    :return: handle
    """
    
    x = fgrid.x
    y = fgrid.y
    nv = fgrid.nvMin0
    
    if Global or fgrid.type == 'Global':
        edge_cell = np.where(np.max(x[nv], axis=1) - np.min(x[nv], axis=1) > 181)[0]
        nv = np.delete(nv, edge_cell, axis=0)
    
    tri_nv = tri.Triangulation(x, y, nv.T)
    
    # plt.figure()
    h = plt.triplot(tri_nv, 'k-', lw=0.5)
    plt.setp(h, color=Color)
    # plt.show()
    
    return h


def f_2d_image(fgrid, var):
    """
    draw 2d image
    :param fgrid: FVCOM grid
    :param var: variable
    :return: handle
    """
    
    def changem(array, new_vals, old_vals):
        """将 array 中的 old_vals 替换为 new_vals"""
        mapping = dict(zip(old_vals, new_vals))
        return np.vectorize(mapping.get, otypes=[array.dtype])(array)
    
    MaxLon = fgrid.MaxLon
    MinLon = MaxLon - 360.
    MidLon = MaxLon - 180.
    
    x = fgrid.x
    y = fgrid.y
    nv = fgrid.nvMin0
    node = fgrid.node
    nele = fgrid.nele
    
    if var.size == node:
        pass
    else:
        if var.size == var.shape[0]:
            var = var[:, 0]
        elif var.size == var.shape[1]:
            var = var[0]

    if var.size == node:
        pass
    elif var.size == nele:
        raise UserWarning('Not implemented yet !!!')
    else:
        raise ValueError('Wrong size of variable !!!')
    
    if fgrid.type == 'Global':
        # 1. 找到 y == 90 的节点 (北极点)
        Pole_node = np.where(y == 90.)[0]
        # 2. 计算三角形单元的最大和最小经度
        max_cell_x = np.max(x[nv], axis=1)
        min_cell_x = np.min(x[nv], axis=1)
        # 3. 找到满足条件的边界单元
        edge_cell = (max_cell_x > MidLon) & (min_cell_x < MidLon) & ((max_cell_x - min_cell_x) > 180.)
        # 4. 提取边界单元和边界节点
        edge_nv = nv[edge_cell]
        edge_node = np.setdiff1d(np.unique(edge_nv), Pole_node)
        # 5. 处理边界节点的经度
        edge_x = x[edge_node]
        k1 = np.where(edge_x < MidLon)[0]
        k2 = np.where(edge_x > MidLon)[0]
        edge_x[k1] += 360.0
        edge_x[k2] -= 360.0
        # 6. 处理边界节点的纬度
        edge_y = y[edge_node]
        # 7. 修改边界单元中的节点索引
        edge_nv_right = changem(edge_nv, k1 + node, edge_node[k1])
        edge_nv_left = changem(edge_nv, k2 + node, edge_node[k2])
        # 8. 扩展节点和单元信息
        x = np.concatenate((x, edge_x))
        y = np.concatenate((y, edge_y))
        nv = np.vstack((nv[~edge_cell], edge_nv_right, edge_nv_left))
        var = np.concatenate((var, var[edge_node]))
    else:
        pass
    tri_nv = tri.Triangulation(x, y, nv.T)
    
    # plt.interactive(False)
    # plt.figure()
    h = plt.tripcolor(tri_nv, var)
    # plt.colorbar()
    # plt.show()
    
    return h




#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/11/18 23:15
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
https://docs.python.org/3/library/warnings.html
"""

import warnings

warnings.warn("UserWarning", UserWarning)  # --> 默认
warnings.warn("DeprecationWarning", DeprecationWarning)  # --> 弃用警告
warnings.warn("PendingDeprecationWarning", PendingDeprecationWarning)  # --> 将来将弃用警告
warnings.warn("SyntaxWarning", SyntaxWarning)  # --> 语法警告
warnings.warn("RuntimeWarning", RuntimeWarning)  # --> 运行时警告
warnings.warn("FutureWarning", FutureWarning)  # --> 未来警告
warnings.warn("ImportWarning", ImportWarning)  # --> 导入警告
warnings.warn("UnicodeWarning", UnicodeWarning)  # --> Unicode警告
warnings.warn("BytesWarning", BytesWarning)  # --> 字节警告
warnings.warn("ResourceWarning", ResourceWarning)  # --> 资源警告
warnings.warn("Warning", Warning)  # --> 警告



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/7/20 19:56
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
下载记录器
"""
import json
import os
from collections import OrderedDict

from .. import Blog


class deal_download_json:
    def __init__(self, file_all, json_file, logger=Blog().logger, **kwargs):
        self.file_all = file_all
        self.json_file = json_file
        self.logger = logger
        self.kwargs = kwargs
    
    def operate_json(self):  # sourcery skip: low-code-quality
        """
        生成json文件，里面的key是数据文件名以及一个download_sign，value为0或1或2
        0 --> 未下载
        1 --> 已下载
        2 --> 已转换
        """
        if not os.path.exists(self.json_file):  # 不存在先制作一个列表，里面的元素是要下载数据文件名
            file_need_down, file_need_trans = self.generate_json()
        else:  # 存在检查是否全部下载完成
            file_need_down, file_need_trans = self.regenerate_json()
        return file_need_down, file_need_trans
    
    def generate_json(self):
        """
        不存在json文件，生成json文件
        """
        download_sign = OrderedDict()  # 有序字典
        download_sign.update(self.kwargs)
        # 不存在先制作一个列表，里面的元素是要下载数据文件名
        file_need_down = []  # 要下载的文件名列表
        file_need_trans = []  # 要转换的文件名列表
        # 生成文件名
        for filename in self.file_all:
            download_sign[filename] = 0
            file_need_down.append(filename)
        with open(self.json_file, "w", encoding='utf-8') as f:
            json.dump(download_sign, f, indent=2, sort_keys=False, ensure_ascii=False)  # 如果sort_keys=True,则为排序
        self.logger.debug(f'初始{os.path.basename(self.json_file)}文件已生成')
        return file_need_down, file_need_trans
    
    def regenerate_json(self):
        """
        存在json文件，读取json文件
        """
        with open(self.json_file, 'r', encoding='utf-8') as f:
            data_ori = json.load(f)
        file_list_ori = [k for k in data_ori.keys() if data_ori[k] in [1, 2]]
        file_list_plus = list(set(file_list_ori) - (set(file_list_ori) & set(self.file_all)))
        file_list_plus = sorted(file_list_plus, key=lambda x: file_list_ori.index(x))
        
        data_new = OrderedDict()
        data_new.update(self.kwargs)
        data_new_key = self.file_all + file_list_plus
        for kkey in data_new_key:
            data_new[kkey] = data_ori[kkey] if kkey in data_ori.keys() else 0
        if 0 in data_new.values():
            self.logger.debug('开始补充下载')
        elif 1 in data_new.values():
            self.logger.debug('开始补充转化')
        else:
            self.logger.debug('当前没有文件需要下载或转化')
        file_need_down = [k for k, v in data_new.items() if v == 0]
        file_need_trans = [k for k, v in data_new.items() if v == 1]
        with open(self.json_file, "w", encoding='utf-8') as f:
            json.dump(data_new, f, indent=2, sort_keys=False, ensure_ascii=False)  # 如果sort_keys=True,则为排序
        self.logger.debug(f'更新{os.path.basename(self.json_file)}文件')
        return file_need_down, file_need_trans
    
    def update_json(self, file, status_code):
        """
        更新json文件中的value
        :param file: 文件名 --> gfs.t00z.pgrb2.0p25.f001
        :param status_code: 状态码 --> 0:未下载 1:已下载 2:已转化
        """
        with open(self.json_file, 'r', encoding='utf-8') as f:
            data_all = json.load(f)
        data_all[file] = status_code
        with open(self.json_file, "w", encoding='utf-8') as f:
            json.dump(data_all, f, indent=2, sort_keys=False, ensure_ascii=False)
        self.logger.debug(f'已更新json文件中{file}的状态！')



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/3/9 14:06
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
https://pypi.org/project/Alaea/#history
"""
import os

# /Users/christmas/opt/anaconda3/lib/python3.9/site-packages

PROJECT = 'Alaea'
os.chdir(f'/Users/christmas/Documents/Code/Project/A-pypi/{PROJECT}')
os.system('rm -rf dist build')
os.system('python3 setup.py sdist bdist_wheel')
os.system('twine upload dist/*')
os.system(f'rm -rf /Users/christmas/Documents/Code/Project/A-pypi/{PROJECT}/{PROJECT}.egg-info')
os.system(f'rm -rf /Users/christmas/Documents/Code/Project/A-pypi/{PROJECT}/dist')
os.system(f'rm -rf /Users/christmas/Documents/Code/Project/A-pypi/{PROJECT}/build')
os.system(f'rm -rf /Users/christmas/Documents/Code/Project/A-pypi/{PROJECT}/{PROJECT}.egg-info')



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2025/1/9 11:07
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
 
"""
import numpy as np

__all__ = ['calc_xcyc', 'check_grid_type', 'f_calc_nbve']


def calc_xcyc(lon, lat, nv, Global=False):
    """
    计算三角形的质心坐标
    :param lon: 三角形的经度 {ndarray(node, )}
    :param lat: 三角形的纬度 {ndarray(node, )}
    :param nv: 三角形的节点 {ndarray(nele, 3)}
    :param Global: 是否是全球网格
    :return: lonc, latc
    """
    if nv.shape[0] == 3:  # 如果nv第0维度是3，则转置
        nv = nv.T
    if np.min(nv) == 1:  # 如果nv从1开始，则减1
        nv = nv - 1
    if Global:
        r = 90.0 - lat[nv]
        theta = lon[nv]
        x = r * np.cos(np.deg2rad(theta))
        y = r * np.sin(np.deg2rad(theta))
        
        xc = np.mean(x, axis=1)
        yc = np.mean(y, axis=1)
        rc = np.sqrt(xc ** 2 + yc ** 2)
        thetac = np.degrees(np.arctan2(yc, xc))
        lonc = thetac
        latc = 90.0 - rc
    else:
        lonc = np.mean(lon[nv], axis=1)
        latc = np.mean(lat[nv], axis=1)

    return lonc, latc


def check_grid_type(lon, lat, eps=0.11):
    """
    检查网格类型
    :param lon: 经度
    :param lat: 纬度
    :param eps: 误差
    :return: 'Global' or 'Regional'
    """
    
    ftype = 'Regional'
    
    xlims = [np.min(lon), np.max(lon)]
    ylims = [np.min(lat), np.max(lat)]
    
    if ylims[0] >= -90 and np.abs(ylims[1] - 90) < eps and np.abs(xlims[0]) < eps and np.abs(xlims[1] - 360) < eps:
        if np.all(np.histogram(lon, bins=np.arange(0, 360, 5))[0] > 0):
            ftype = 'Global'
    elif ylims[0] >= -90 and np.abs(ylims[1] - 90) < eps and np.abs(xlims[0] + 180) < eps and np.abs(xlims[1] - 180) < eps:
        if np.all(np.histogram(lon, bins=np.arange(-180, 180, 5))[0] > 0):
            ftype = 'Global'

    return ftype


def f_calc_nbve(fgrid):
    """
    Calculate nbve (cell id around each node)
    :param fgrid: 网格数据
    :return: nbve
    """
    
    nv = fgrid.nvMin0
    node = fgrid.node
    nele = fgrid.nele
    
    # 将 nv 转换为 nv_l，类似于 MATLAB 的操作
    nv_l = np.hstack((
        nv.T.reshape(nele * 3, 1),  # 转置后拉平成一列
        np.repeat(np.arange(1, nele + 1), 3).reshape(-1, 1)  # 单元编号，每个单元重复 3 次
    ))
    # 按照 nv_l 的第一列（节点编号）排序
    nv_l = nv_l[np.argsort(nv_l[:, 0])]
    node_center = nv_l[:, 0]  # 节点编号
    cell_around = nv_l[:, 1]  # 单元编号
    # 计算每个节点的单元连接数
    counts = np.histogram(node_center, bins=np.arange(1, node + 2))[0]
    # 找到最大连接单元数
    max_ntve = np.max(counts)
    # 累积连接数
    counts_accum = np.concatenate(([0], np.cumsum(counts[:-1])))
    # 构造索引
    index = ((node_center - 1) * max_ntve - counts_accum[node_center - 1] + np.arange(1, nele * 3 + 1)).astype(int) - 1
    # 初始化 nbve，默认值是 nele + 1
    nbve = np.full((node, max_ntve), nele + 1, dtype=int)
    # 填充 nbve
    nbve.flat[index] = cell_around
    # 转置为节点优先的形式
    nbve = nbve.T
    return nbve

    


#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/4/23 下午9:42
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
Reference: https://www.bilibili.com/read/cv6944361/
"""

import numpy as np
from netCDF4 import Dataset

_all_ = ['convert_nc22dm', 'write_2dm']


def convert_nc22dm(_fnc: str, _f2dm: str):
    with Dataset(_fnc) as f1:
        lon = f1.variables['lon'][:]
        lat = f1.variables['lat'][:]
        nv = f1.variables['nv'][:]
        dep = f1.variables['h'][:]
    write_2dm(_f2dm, lon, lat, nv, dep)


def write_2dm(_f2dm, _x, _y, _nv, _h=None):  # sourcery skip: extract-method
    if _h is None:
        _h = np.zeros(_x.shape)
    # regular transport content
    elehd = np.tile(['E3T'], (1, _nv.shape[1]))  # build 'E3T' string as head
    ndhd = np.tile(['ND'], _x.shape)  # build 'E3T' string as head
    #  E3T nvnum nv1 nv2 nv3 1
    e3t = np.vstack((elehd, np.arange(_nv.shape[1])+1, _nv, np.ones(_nv.shape[1], dtype=int)))
    #  ND ndnum lon lat depth
    nd = np.vstack((ndhd, np.arange(_x.shape[0])+1, _x, _x, _h))
    e3t = np.transpose(e3t)
    nd = np.transpose(nd)
    #create 2dm
    with open(_f2dm, 'w') as f2:
        f2.write('MESH2D\n')
        f2.write('MESHNAME "Mesh"\n')
        for i in range(e3t.shape[0]):
            f2.write(' '.join(e3t[i, :])+'\n')
        for j in range(nd.shape[0]):
            f2.write(' '.join(nd[j, :])+'\n')
        f2.write('BEGPARAMDEF'+'\n')
        f2.write('GM  "Mesh"'+'\n')
        f2.write('SI  0'+'\n')
        f2.write('DY  0'+'\n')
        f2.write('TU  ""'+'\n')
        f2.write('TD  0  0'+'\n')
        f2.write('NUME  3'+'\n')
        f2.write('BCPGC  0'+'\n')
        f2.write('BEDISP  0 0 0 0 1 0 1 0 0 0 0 1'+'\n')
        f2.write('BEFONT  0 2'+'\n')
        f2.write('BEDISP  1 0 0 0 1 0 1 0 0 0 0 1'+'\n')
        f2.write('BEFONT  1 2'+'\n')
        f2.write('BEDISP  2 0 0 0 1 0 1 0 0 0 0 1'+'\n')
        f2.write('BEFONT  2 2'+'\n')
        f2.write('ENDPARAMDEF'+'\n')
        f2.write('BEG2DMBC'+'\n')
        f2.write('MAT  1 "material 01"'+'\n')
        f2.write('END2DMBC'+'\n')



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/7/20 19:45
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
    用于存放参数的类
    pip install python-box
    from box import Box
"""

__all__ = [
    'P_para'
]


class P_para:
    def __init__(self, _dict=None, **kwargs):
        """
        用于存放参数的类
        :param _dict: 传入字典
        :param kwargs: 传入参数
        """
        if not _dict:
            self.__dict__.update(kwargs)
        else:
            self.__dict__ = _dict
    
    def __repr__(self):
        # return str(self.__dict__)
        return str(self.__class__.__name__)
    
    def addParam(self, **kwargs):
        self.__dict__.update(kwargs)
    
    def getParam(self, key):
        return self.__dict__[key]
    
    def getParams(self):
        return self.__dict__
    
    def delParam(self, key):
        del self.__dict__[key]
        
    def keys(self):
        return self.__dict__.keys()
    
    @property
    def key(self):
        return self.__dict__.keys()



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/11/6 16:32
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

https://mp.weixin.qq.com/s/UDBQYQgErxUxNJvWc2E1LA
"""

import warnings

from osgeo import ogr

warnings.warn('This module has not been checked yet', UserWarning)


def convert_kml_to_shp(in_file, out_file):
    """
    Converts a KML file to a shapefile using GDAL/OGR.

    Args:
        in_file (str): Path to the input KML file.
        out_file (str): Path to the output SHP file.
    """
    # Open the KML file using OGR
    ds_in = ogr.Open(in_file)
    if ds_in is None:
        print(f"Failed to open {in_file}")
        return

    layer = ds_in.GetLayer(0)

    # Get the spatial reference of the source layer
    srs = layer.GetSpatialRef()

    # Create the output shapefile
    ds_out = ogr.GetDriverByName('ESRI Shapefile').CreateDataSource(out_file)
    if ds_out is None:
        print(f"Failed to create {out_file}")
        return

    # Copy the layer definition
    layer_defn = layer.GetLayerDefn()
    layer_out = ds_out.CreateLayer('output', srs=srs, geom_type=layer_defn.GetGeomType())

    # Copy the fields
    for i in range(layer_defn.GetFieldCount()):
        field_defn = layer_defn.GetFieldDefn(i)
        layer_out.CreateField(field_defn)

    # Get the Feature Definition for the output layer
    feat_defn = layer_out.GetLayerDefn()

    # Process each feature in the input layer
    for feat_in in layer:
        # Create a new feature
        feat_out = ogr.Feature(feat_defn)

        # Set the geometry
        feat_out.SetGeometry(feat_in.GetGeometryRef())

        # Set the attributes
        for i in range(feat_defn.GetFieldCount()):
            feat_out.SetField(feat_defn.GetFieldDefn(i).GetNameRef(), feat_in.GetField(i))

        # Create the feature in the output layer
        layer_out.CreateFeature(feat_out)

        # Clean up
        feat_out = None

    # Clean up
    ds_out = None
    ds_in = None

    print(f"{in_file} converted to {out_file}")


if __name__ == '__main__':
    shapefile = r'D:\temp\create_shp_by_fiona1.shp'
    kml_file = r'D:\temp\create_shp_by_fiona.kml'
    convert_kml_to_shp(kml_file, shapefile)



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/4/23 下午9:41
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
from .fvcom_class import *
from .fvcom_image import *
from .fvcom_load import *
from .fvcom_tools import *
from .writeFile import *


def example_pFVCOM():
    """
    Example for pFVCOM
    """
    print('This is an example for pFVCOM')
    fin = '/Users/christmas/Desktop/exampleNC/SCS_avg_0001.nc'
    fgrid = f_load_grid(fin)
    Ttimes = f_load_time(fin)
    ncid = Dataset(fin, 'r')
    zeta = ncid.variables['zeta'][:].data
    u = ncid.variables['u'][:].data
    v = ncid.variables['v'][:].data
    ncid.close()
    plt.figure()
    f_2d_image(fgrid, zeta)
    f_2d_mesh(fgrid, Color='b')
    ind = range(0, fgrid.nele, 5)
    plt.quiver(fgrid.xc[ind], fgrid.yc[ind], u[0, 0, ind], v[0, 0, ind])
    ax = plt.gca()
    ax.set_aspect('equal')
    plt.xlabel('Longitude (degrees)')
    plt.ylabel('Latitude (degrees)')
    plt.title('zeta (m)')
    plt.colorbar()
    plt.show()
    print('End of example')


#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/3/25 11:23
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import contextlib
import fractions
import re
import unicodedata  # 处理ASCii码的包

import numpy as np

from .commonCode import split_path
from .cprintfs import osprint

__all__ = ['read_conf']


# 从file中读取配置信息
def read_conf(_config_file, ele=None):
    conf = {}
    key_same_num = 1
    with open(_config_file, 'r', encoding='utf-8') as f:
        for line in f:
            # 去除空格和注释
            line = line.strip()
            if line.startswith('#') or line.startswith('!') or line.startswith('//') or line.startswith(';'):
                continue
            if line == '':
                continue

            key, value = line.split('=', maxsplit=1)
            key = key.strip()
            value = value.strip()
            value = value.split('#')[0].strip().strip("'").strip('"')
            value = split_path(value)
            [key, value] = char_fill_dic(key, value)
            if value == '':
                value = None
            # 如果conf中已经存在key，则跳过
            if key in conf:
                conf[f'{key}_{key_same_num}'] = value
                key_same_num += 1
            else:
                key_same_num = 1
                conf[key] = value
    try:
        if ele is not None:
            conf = conf[ele]
    except KeyError:
        osprint(f'"{ele} is not in the config file"')
    return conf


def char_fill_dic(key, _str):  # sourcery skip: low-code-quality
    # 不区分大小写比较
    if _str.lower() == '.true.':
        _str = True
    elif _str.lower() == '.false.':
        _str = False
    elif _str.startswith('['):
        _str_1 = _str.split(',')
        _str = _str[1:-1].split(',')
        _str = [x for x in _str if x.strip()]
        try:
            for i in range(len(_str)):
                if is_number(_str[i]):
                    _str[i] = float(_str[i])
                else:
                    tmp_1 = _str[i].strip().split(':')
                    if '/' in tmp_1[0]:  # 否则会出现 误判
                        try:
                            tmp_1 = [float(fractions.Fraction(x)) for x in tmp_1]  # '1/2' -> 0.5
                        except ValueError:  # (1/24) -> 0.041666666666666664
                            tmp_1 = [float(eval(x)) for x in tmp_1]
                    _str[i] = np.arange(float(tmp_1[0]), float(tmp_1[2]) + float(tmp_1[1]), float(tmp_1[1])).tolist()  # [1:1,120]
            _str = flatten_list(_str, [])
        except ValueError:
            try:
                if isinstance(_str[0], str):  # type(_str[0]) == str:
                    _str = [_str[i].replace('[', '').replace(']', '').split() for i in range(len(_str))]
                    with contextlib.suppress(ValueError):
                        _str = [float(x) for _str in _str for x in _str]  # ['1', '2', '3'] -> [1, 2, 3]
                    with contextlib.suppress(TypeError):
                        _str = [x.strip("'").strip('"') for _str in _str for x in _str]  # [["'hs'"], ["'t02'"]] -> ['hs', 't02']
            except IndexError:
                for i in range(len(_str)):
                    _str[i] = _str[i].strip().strip("'").strip('"')
        except IndexError:  # [1,120]
            _str = _str[0].strip().strip("'").strip('"')
            tmp_1 = _str.split(':')
            _str = np.arange(float(tmp_1[0]), float(tmp_1[1])+1).tolist()  # [1:1,120]
    elif _str.startswith('{'):
        # "{'KK' :'sds', 'YY' : 'asd'}" ->  {'KK' :'sds', 'YY' : 'asd'}
        _str = _str[1:-1].split(',')
        _str = {i.split(':')[0].strip().strip("'").strip('"'): i.split(':')[1].strip().strip("'").strip('"') for i in _str}
    elif _str.startswith("'") or _str.startswith('"'):
        _str = _str.strip().strip("'").strip('"')
    elif is_number(_str):
        _str = float(_str)
    else:
        with contextlib.suppress(IndexError):
            if bool(re.match(r'^[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?$', _str.strip()[0])):
                with contextlib.suppress(ValueError):
                    _str = [float(i) for i in _str.split()]
    key = 'None' if key == '' else key
    return key, _str


def is_number(s):
    with contextlib.suppress(ValueError):
        float(s)
        return True
    with contextlib.suppress(TypeError, ValueError):
        unicodedata.numeric(s)  # 把一个表示数字的字符串转换为浮点数返回的函数
        return True
    return False


def flatten_list(_lst, flattened_lst):
    # sourcery skip: default-mutable-arg
    for item in _lst:
        if isinstance(item, list):
            flatten_list(item, flattened_lst)
        else:
            flattened_lst.append(item)
    return flattened_lst



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2022/12/2 00:10
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import contextlib
import getpass
import os
import socket

import requests

__all__ = ['get_free_core', 'get_serve_info', 'get_server_ip_local', 'get_server_ip_public', 'get_serve_ip',
           'grep_from_top', 'grep_from_top_mpi']


def grep_from_top(_exe):  # sourcery skip: inline-immediately-returned-variable
    """
    从top命令中获取进程信息
    :param _exe: 进程名
    :return:
    """
    # ps -ef | grep top | grep -v grep | awk '{print $2}'
    script = f'ps -ef  | grep {_exe}' + ' | grep -v  grep | awk \'{print $2}\''
    pid = os.popen(script).read()
    return pid


def grep_from_top_mpi(_exe):
    """
    从top命令中获取进程信息
    :param _exe: 进程名
    :return:
    """
    # ps -ef | grep top | grep -v grep | awk '{print $2}'
    script = f'ps -ef  | grep {_exe}' + ' | grep -v  grep | awk \'{print $2}\' | wc -l'
    mpi_num = int(os.popen(script).read())
    if mpi_num != 0:
        mpi_num -= 1
    return mpi_num


def get_free_core():
    """
    获取空闲可用的cpu数
    :return: 空闲可用的cpu数
    """
    cpu_num = os.popen("echo $(grep processor /proc/cpuinfo | wc -l)").read()
    used_cpu_percent = os.popen("echo $(top -n 1 -b | grep Cpu | awk '{print $2}')").read()
    used_cpu_num = int(float(used_cpu_percent) / 100 * int(cpu_num))
    cpu_num = int(cpu_num)
    free_cpu_num = cpu_num - used_cpu_num
    return cpu_num, free_cpu_num


def get_serve_info():
    """
    获取服务器信息
    :return: 服务器信息
    """
    user = getpass.getuser()
    hostname = socket.gethostname()
    ip = socket.gethostbyname(hostname)
    return user, hostname, ip


def get_server_ip_local():
    """
    获取服务器内网ip
    :return: 服务器内网ip
    """
    return socket.gethostbyname(socket.gethostname())


def get_server_ip_public():
    """
    获取服务器公网ip
    :return: 服务器公网ip
    """
    # https://ident.me
    # https://ifconfig.me/ip
    # http://icanhazip.com
    # https://checkip.amazonaws.com
    # http://jsonip.com/
    # http://ip.jsontest.com/
    # http://www.trackip.net/ip?json
    _public_ip = None
    with contextlib.suppress(Exception):
        _public_ip = requests.get('https://ident.me').text.strip()
    return _public_ip


def get_serve_ip(_area='ALL'):
    """
    获取服务器ip
    :return: 服务器ip
    """
    if _area == 'ALL':
        return {'local': get_server_ip_local(), 'public': get_server_ip_public()}
    elif _area == 'local':
        return get_server_ip_local()
    elif _area == 'public':
        return get_server_ip_public()



#!/bin/env python3
# -*- coding: utf-8 -*-
#  日期 : 2025/12/23 14:28
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import json


def read_json(_filepath):
    """
    读取JSON文件
    :param _filepath: JSON文件路径
    :return: 解析后的数据
    """
    with open(_filepath, 'r', encoding='utf-8') as f:
        data = json.load(f)
    return data


# 列表去重
def unique_list(_lst):
    """
    列表去重，保持顺序
    :param _lst: 输入列表
    :return: 去重后的列表
    """
    _set = set()
    result = []
    for item in _lst:
        if item in _set or len(item.strip())==0:  # 去重 空行 注释行
            continue
        _set.add(item)
        result.append(item)
    
    return result


# -*- coding: utf-8 -*-
#  日期 : 2022/11/30 11:33
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""

import contextlib
import datetime
import os
import shutil
import sys
import time
import warnings

import numpy as np

__all__ = ['convertToTime', 'new_filename', 'get_date', 'makedirs', 'rmfiles', 'rmtree', 'rmdirs',
           'movefiles', 'split_path', 'timer', 'whether_instanced', 'checkOS']


def convertToTime(strDate=None, listDate=None, _input_format=None, tzinfo=datetime.timezone.utc):
    """
    将 %Y%m%d 格式的8位字符串，转换为日期
    :param strDate: %Y%m%d 格式的8位日期字符串
    :param listDate: %Y%m%d 格式的8位日期字符串列表
    :param _input_format: 输入日期格式
    :param tzinfo: 时区
    :return: datetime 类型的日期
    """
    date = datetime.datetime.now()  # 默认取当天日期

    if strDate is not None and isinstance(strDate, str):
        with contextlib.suppress(Exception):
            if _input_format == 'WRF_output':
                date = datetime.datetime.strptime(strDate, "%Y-%m-%d_%H:%M:%S")  # 2022-11-09_01:00:00
            elif len(strDate) == 8:
                date = datetime.datetime.strptime(strDate, "%Y%m%d")
            elif len(strDate) == 10:
                date = datetime.datetime.strptime(strDate, "%Y%m%d%H")
            elif len(strDate) == 12:
                date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M")
            elif len(strDate) == 14:
                date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M%S")
            elif len(strDate) == 19:
                if strDate[10] == ' ':
                    date = datetime.datetime.strptime(strDate, "%Y-%m-%d %H:%M:%S")  # 2022-11-09 01:00:00
                elif strDate[10] == '_':
                    date = datetime.datetime.strptime(strDate, "%Y-%m-%d_%H:%M:%S")  # 2022-11-09_01:00:00
                elif strDate[10] == 'T':
                    date = datetime.datetime.strptime(strDate, "%Y-%m-%dT%H:%M:%S")  # 2022-11-09T01:00:00
        date = date.replace(tzinfo=tzinfo)
    elif listDate is not None and isinstance(listDate, list):
        with contextlib.suppress(Exception):
            date = [convertToTime(i, _input_format) for i in listDate]
    elif isinstance(strDate, str) is False and isinstance(listDate, list) is False:
        raise TypeError('strDate must be str or listDate must be list')
    return date


def new_filename(_pre, _lon, _lat, _date, _res):
    """
    根据前缀、经纬度、日期、分辨率生成输出文件名
    :param _pre: 输出文件前缀
    :param _lon: 经度
    :param _lat: 纬度
    :param _date: 日期
    :param _res: 分辨率
    :return: 输出文件名
    """
    if np.min(_lon) < 0:
        lon_1 = str(format(abs(np.min(_lon)), '.2f')).zfill(6) + 'W'
    else:
        lon_1 = str(format(abs(np.min(_lon)), '.2f')).zfill(6) + 'E'
    if np.max(_lon) < 0:
        lon_2 = str(format(abs(np.max(_lon)), '.2f')).zfill(6) + 'W'
    else:
        lon_2 = str(format(abs(np.max(_lon)), '.2f')).zfill(6) + 'E'
    if np.min(_lat) < 0:
        lat_1 = str(format(abs(np.min(_lat)), '.2f')).zfill(5) + 'S'
    else:
        lat_1 = str(format(abs(np.min(_lat)), '.2f')).zfill(5) + 'N'
    if np.max(_lat) < 0:
        lat_2 = str(format(abs(np.max(_lat)), '.2f')).zfill(5) + 'S'
    else:
        lat_2 = str(format(abs(np.max(_lat)), '.2f')).zfill(5) + 'N'
    filename = f'{_pre}_{lon_1}_{lon_2}_{lat_1}_{lat_2}_{str(_date)}_{str(_res)}.nc'
    del lon_1, lon_2, lat_1, lat_2
    return filename


def get_date():
    """
    获取日期
    :return:
    """
    date = ''
    if len(sys.argv) == 1:
        date = datetime.datetime.now().strftime("%Y%m%d")
    elif len(sys.argv) >= 2 and len(sys.argv[1]) == 8:
        date = sys.argv[1]
    return date


def makedirs(*path, exist_ok=False):
    """
    创建文件夹
    :param path: 文件夹路径
    :param exist_ok: 是否忽略错误
    :return:
    """
    for p in path:
        if not os.path.exists(p):
            os.makedirs(p, exist_ok=exist_ok)


def rmfiles(*path):
    """
    删除文件
    :param path: 文件路径
    :return:
    """
    for p in path:
        try:
            if os.path.exists(p):
                os.remove(p)
        except PermissionError:
            warnings.warn(f'Maybe {p} is a directory, use rmtree() instead', SyntaxWarning)


def rmtree(*path, ignore_errors=False, onerror=None):
    """
    删除文件夹及包含的所有文件
    :param path: 文件夹路径
    :param ignore_errors: 是否忽略错误
    :param onerror: 错误处理
    :return:
    """
    for p in path:
        if os.path.exists(p):
            shutil.rmtree(p, ignore_errors=ignore_errors, onerror=onerror)


def rmdirs(*path):
    """
    删除文件夹
    :param path: 文件夹路径
    :return:
    """
    for p in path:
        try:
            if os.path.exists(p):
                os.removedirs(p)
        except OSError:
            warnings.warn(f'Maybe {p} is not empty, use rmtree() instead', SyntaxWarning)


def movefiles(*src, dst, cover=True):
    """
    移动文件
    :param src: 源文件路径
    :param dst: 目标文件路径
    :param cover: 是否覆盖
    :return: 0
    """
    for s in src:
        if os.path.exists(s):
            if cover:
                rmtree(dst)
            shutil.move(s, dst)
    
    return 0


def split_path(_path, _split='/'):
    """
    如果路径名最后一位是'/'，则去掉
    :param _path: 路径名
    :param _split: 分隔符
    :return: 路径名
    """
    with contextlib.suppress(IndexError):
        if _path[-1] == _split:
            _path = _path[:-1]
    return _path


def timer(func):
    def inside(self):
        t1 = time.time()
        func(self)
        t2 = time.time()
        print('task time:{:.2f}s'.format(t2 - t1))

    return inside


def whether_instanced(_class):
    """
    判断是否被实例化
    :param _class: 类名
    """
    has_instance = False
    instanced = {}
    instances = globals().copy()

    for var_name, var_value in instances.items():
        if isinstance(var_value, _class):
            has_instance = True
            instanced[var_name] = var_value

    return has_instance, instanced


def checkOS(_os=None):
    """
    检查操作系统
    :param _os: 操作系统
    :return: 操作系统 LNX WIN MAC
    """
    if sys.platform == 'linux':
        OS = 'LNX'
    elif sys.platform == 'win32':
        OS = 'WIN'
    elif sys.platform == 'darwin':
        OS = 'MAC'
    else:
        OS = 'UNKNOW'
    
    if _os is not None:
        if _os == OS:
            return True
        else:
            return False
    else:
        return OS


#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2025/10/23 16:59
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
from datetime import datetime
from typing import List, Union

import numpy as np
import pandas as pd
import xarray as xr

from .. import Ttime
from .xr_attrs import (add_ice_attrs, add_lat_attrs, add_lon_attrs,
                       add_time_attrs, add_TIME_attrs, fixed_encoding)


def xr_ice_dateset(_lon: np.ndarray,
                   _lat: np.ndarray,
                   _time: Union[List, str, np.ndarray, datetime, np.datetime64, pd.Timestamp],
                   **kwargs):
    """
    生成海冰数据的xarray.Dataset
    :param _lon: 经度
    :param _lat: 纬度
    :param _time: 时间
    :param kwargs: aice, tive
                    engine, dtype, complevel, zlib, shuffle, fletcher32, contiguous,
                    product_name, source, start
        aice: sea ice concentration (%) 海冰密集度
        tice: sea ice thickness (m) 海冰厚度
        engine: h5netcdf(default), netcdf4, scipy
        dtype: float32(default), float64
        complevel: 4(default) 0-9
        zlib: True(default)
        shuffle: True(default)
        fletcher32: True(default)
        contiguous: True(default)
        product_name: None(default) 产品名称
        source: Alaea(default) 数据来源
        start: None(default) 数据起始时间
    :return: xarray.Dataset
    """
    program_version = 'V1.0'
    history = 'Created by Python' + ' at ' + datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    if _time[0].__class__ == str:
        Ttimes = Ttime(TIME=_time)
    elif _time[0].__class__ == datetime or _time[0].__class__ == np.datetime64 or _time[0].__class__ == pd.Timestamp:
        Ttimes = Ttime(Times=_time)
    else:
        raise ValueError(_time[0].__class__)
    
    xr_ice = xr.Dataset(
        data_vars={
            "TIME": ("time", Ttimes.TIME)},
        coords={
            "longitude": _lon,
            "latitude": _lat,
            "time": Ttimes.time,
        },
        attrs={
            "product_name": kwargs.get('product_name', 'None'),
            "source": kwargs.get('source', 'Alaea'),
            "start": kwargs.get('start', 'None'),
            "history": history,
            "program_version": program_version,
        }
    )
    xr_ice = add_lon_attrs(xr_ice, lon_name='longitude')
    xr_ice = add_lat_attrs(xr_ice, lat_name='latitude')
    xr_ice = add_time_attrs(xr_ice, time_name='time')
    xr_ice = add_TIME_attrs(xr_ice, TIME_name='TIME', dim_name='DateStr')
    
    if 'aice' in kwargs:
        _aice = kwargs.get('aice')
        xr_ice['aice'] = (("time", "latitude", "longitude"), _aice)
    
    if 'tice' in kwargs:
        _tice = kwargs.get('tice')
        xr_ice['tice'] = (("time", "latitude", "longitude"), _tice)

    xr_ice = add_ice_attrs(xr_ice)
    
    # 去掉上面用过的key
    for key in ['aice', 'tice',
                'product_name', 'source', 'start']:
        if key in kwargs:
            kwargs.pop(key)
    xr_ice = fixed_encoding(xr_ice, **kwargs)
    
    # 排序
    ordered_vars = ['longitude', 'latitude', 'time', 'TIME',
                    'aice', 'tice']
    existing_vars = [var for var in ordered_vars if var in xr_ice.variables]
    xr_ice = xr_ice[existing_vars]
    
    return xr_ice



# -*- coding: utf-8 -*-

# @File    : shell_try.py
# @Date    : 2023-06-26
# @Author  : Dovelet
# -*- coding: utf-8 -*-

# @File    : shell_python.py
# @Date    : 2023-06-25
# @Author  : Dovelet
import os
import re
from platform import system

from .. import Blog

__all__ = ['RS_file', 'RS_example', 'read_env_sh', 'source_env']

logger_RS = Blog(_loger_name='ReadSourceENV').logger


def filter_dict(_pattern, _char, _dict):
    """
    :param _pattern: 正则表达式
    :param _char: 要替换的字符
    :param _dict: 要处理的字典
    """
    for kkey, vvalue in _dict.items():
        _alter = re.findall(_pattern, vvalue)
        for _al in _alter:
            _rekey = _al.strip(_char)
            if _rekey in _dict.keys():
                vvalue = vvalue.replace(_al, _dict[_rekey])
                _dict[kkey] = vvalue
    return _dict


def read_env_sh(_Sfile, _logger=logger_RS):  # sourcery skip: low-code-quality
    """
    读取bash.oneapi文件中的加载环境变量的行，并写入os.environ里，
    以 #, if, source开头的行不读取
    非export开头的行不加入os.environ
    """
    # 获取基本路径, 非export开头的行
    path_dict = {}
    with open(_Sfile, 'r') as f:
        for line in f:
            # 跳过以 # 开头的行
            if line.startswith('#') or line.startswith('if') or line.startswith('source') or line.startswith('export'):
                continue
            # 分割键值对
            parts = line.strip().split('=')
            if len(parts) != 2:
                continue
            if '#' in parts[1]:
                parts[1] = parts[1].split('#')[0]
            # 添加键值对
            key, value = parts
            path_dict[key.strip()] = value.strip()

    # 路径的拼接
    pattern1 = r'\$\{[^\}]+\}'  # 匹配${...}的值
    _char = '${}'
    path_dict = filter_dict(pattern1, _char, path_dict)

    # 将export'开头的line一句一句的读取出来
    export_dict = {}
    pattern2 = r'\$([^:]+)'  # 匹配$...的值
    with open(_Sfile, 'r') as f:
        for line in f:
            if line.startswith('export'):
                parts = line.strip().split('=')
                if len(parts) != 2:
                    continue
                if '#' in parts[1]:
                    parts[1] = parts[1].split('#')[0]
                # 检查键是否已经存在
                ex_key, ex_value = parts
                ex_key = ex_key.replace('export', '').strip()  # 去掉export
                ex_value = ex_value.strip()  # 去掉空格
                # 头一次出现的键，添加到字典中
                if ex_key in export_dict:
                    # 多次出现的键，替换字典中的值
                    # 替换${...}
                    alter = re.findall(pattern1, ex_value)
                    for al in alter:
                        rekey = al.strip('${}')
                        if rekey in path_dict.keys():
                            ex_value = ex_value.replace(al, path_dict[rekey])
                    # 替换$...
                    alter = re.findall(pattern2, ex_value)
                    for al in alter:
                        rekey = al.strip('$')
                        if rekey in export_dict:
                            ex_value = ex_value.replace(f'${al}', export_dict[rekey])
                            export_dict[ex_key] = ex_value.strip()
                else:
                    try:
                        # 替换${...}
                        alter = re.findall(pattern1, ex_value)
                        for al in alter:
                            rekey = al.strip('${}')
                            if rekey in path_dict.keys():
                                ex_value = ex_value.replace(al, path_dict[rekey])
                                export_dict[ex_key] = ex_value.strip()
                        # 替换$...
                        alter = re.findall(pattern2, ex_value)
                        for al in alter:
                            # 头一次出现时说明要从环境变量里面取值
                            if al == f'{ex_key}':
                                _logger.debug(f'${al} appears for the first time')
                                env_old = os.environ[ex_key]
                                ex_value = ex_value.replace(f'${al}', env_old)
                                export_dict[ex_key] = ex_value.strip()
                            else:
                                rekey = al.strip('$')
                                if rekey in export_dict:
                                    ex_value = ex_value.replace(f'${al}', export_dict[rekey])
                                    export_dict[ex_key] = ex_value.strip()
                    except KeyError:
                        # 判断系统是windows linux还是mac
                        if system() == 'Windows':
                            _logger.debug(f'KeyError: {ex_key} is not in os.environ')
    # 把通过export添加的路径给替换掉
    export_dict = filter_dict(pattern1, _char, export_dict)
    for key, value in export_dict.items():
        _logger.debug([key, value])
    return export_dict


def source_env(_export_dict):
    """
    :param _export_dict: 读取bash.oneapi文件中的加载环境变量的行，并写入os.environ里，
    :return: 返回一个字典
    """
    for key, value in _export_dict.items():
        os.environ[key] = value


def RS_file(_Sfile, _logger=logger_RS):
    """
    :param _Sfile: bash.oneapi文件的路径
    :param _logger: 日志记录器
    :return: None
    """
    source_env(read_env_sh(_Sfile, _logger))


def RS_example():
    """
    :return: None
    """
    _Sfile = r'D:\Graduate_1\new_grid\move_file\bash.oneapi'
    RS_file(_Sfile, logger_RS)


if __name__ == '__main__':
    RS_example()



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2022/12/1 12:19
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
from .Blogging import *
from .commonCode import *
from .cprintfs import *
# from .pFVCOM import *
from .Pyshell import *
from .read_conf import *
from .S_DateTime import *
from .SameAsMATLAB import *
from .server_info import *
from .Standard import *
from .tools import *



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#  日期 : 2023/3/1 20:22
#  作者 : Dovelet, Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
用于将网格数据转换为单点数据， 并将单点数据转换为json｜txt文件
Have done!!!
"""
import argparse
import contextlib
import datetime
import itertools
import json
import math
import os

import netCDF4
import numpy as np
import scipy.spatial as spt

from Alaea import osprint2

__all__ = ['Grid2Sca']


class Grid2Sca:
    def __init__(self, s_lon, s_lat, _input, _output, _name, value_list, title_list, m_hours=216, time_zone=8, search_point=4, _location=None, _method=None):
        self.s_lon = s_lon  # 单点经度
        self.s_lat = s_lat  # 单点纬度
        self._input = _input  # 输入数据文件夹， 如：'/home/ocean/ForecastSystem/WW3_6.07/Output/ww3_dql/', 内存有'yyyymmdd/ww3.yyyymmdd.nc'文件
        self._output = _output    # 输出数据文件夹， 如：'/home/ocean/PostProcess/160/Data/single_json/'
        self._name = _name          # 输出文件名称， 如：'dql01_'
        self.value_list=value_list  # 需要提取的变量列表
        self.title_list=title_list    # 需要提取的变量名称列表
        self.m_hours = list(range(m_hours))  # 预报时效
        self.time_zone = time_zone    # 时间区间， 如：8
        self.search_point = search_point  # 搜索点数
        self._location = _location  # json文件的location信息
        self.method = _method  # 插值方法

    @staticmethod
    def getTime(_date, _hour):
        """
        获取预报的日期/小时
        :param _date: 任务日期
        :param _hour: 小时
        :return: 日期/小时(10位长度的字符串)
        """
        if _hour == 215:
            _hour = 215

        date = convertToTime(_date)
        date_t = date + datetime.timedelta(hours=int(_hour))

        return date_t.strftime('%Y%m%d%H')

    def makefore(self, _date, _txt=True, _json=True):
        # sourcery skip: extract-duplicate-method, low-code-quality
        """
        txt 预报单的生成
        :param _date: 任务日期
        :param _txt: 是否生成txt文件
        :param _json: 是否生成json文件
        :return: none
        """
        if isinstance(_date, int):
            _date = str(_date)

        file_path = self._output
        if not os.path.exists(file_path):
            os.makedirs(file_path)
        methond, para1, para2 = self.judge_method(_date)  # 并不知道所采用的是什么方法，但都会传回三个参数
        # 写入文件内容----------------------------------begin

        if _txt and not _json:  # 只生成txt文件
            osprint2('INFO', '生成txt文件中')

            tmp = self.m_hours

            file_name = self._name + _date + ".txt"

            file = os.path.join(file_path, file_name)

            with open(file, "w") as fl:
                # 写入文件头----------------------------------begin
                fl.writelines("UTC{0:+03d}".format(self.time_zone).rjust(10, " "))
                for ele in self.title_list:
                    fl.writelines(ele.rjust(14, " "))
                fl.writelines('\n')
                # 写入文件头----------------------------------end

                for i in range(max(tmp) + 1):
                    fc_time = self.getTime(_date, i)  # 获取时间
                    txt_ele, json_ele = self.getfore(_date, i, methond, para1, para2)  # 获取
                    del json_ele
                    if i in self.m_hours:
                        fl.writelines(fc_time)
                        fl.writelines(txt_ele)
                        fl.writelines('\n')
                        osprint2('SUCCESS', f"hour in={str(i)}")
        elif _txt:
            osprint2('INFO', '开始生成txt和json文件')

            tmp = self.m_hours
            json_data_list = []
            txt_data_list = []
            for i in range(max(tmp) + 1):
                # 创建json数据----------------------------------begin
                fc_time = self.getTime(_date, i)  # 第一列日期留空白

                txt_ele, json_ele = self.getfore(_date, i, methond, para1, para2)  # 获取
                del txt_ele
                for j in range(len(json_ele)):
                    if str(json_ele[j]).strip() == "--":
                        json_ele[j]=np.nan

                JsonData, TxtData = self.getTxtJsonData(fc_time, json_ele)  # dict()
                ## json
                json_data_list.append(JsonData)
                ## txt
                txt_data_list.append(TxtData)

                # 创建json数据----------------------------------end
                if i in self.m_hours:
                    osprint2('SUCCESS', f"hour in={str(i)}")
            # 创建json文件----------------------------------begin
            file_name_json = self._name + _date + ".json"
            file_json = os.path.join(file_path, file_name_json)
            if self._location is None:
                new_dict = {"location": {"lon": self.s_lon, "lat": self.s_lat}, "data": json_data_list}
            else:
                new_dict = {"location": self._location, "data": json_data_list}

            with open(file_json, "w") as f:
                json.dump(new_dict, f, ensure_ascii=False, indent=10)
            # 写入json文件内容----------------------------------end

            # 创建txt文件----------------------------------begin
            file_name_txt = self._name + _date + ".txt"
            file_txt = os.path.join(file_path, file_name_txt)
            # 写入txt文件内容----------------------------------begin
            with open(file_txt, "w") as fl:
                fl.writelines("UTC{0:+03d}".format(self.time_zone).rjust(10, " "))
                for ele in self.title_list:
                    fl.writelines(ele.rjust(14, " "))
                fl.writelines('\n')
                for row in txt_data_list:
                    fl.writelines(row)
                    fl.writelines('\n')
            # 写入txt文件内容----------------------------------end
        elif _json:
            osprint2('INFO', '开始生成json文件')

            tmp = self.m_hours
            json_data_list = []
            for i in range(max(tmp) + 1):
                # 创建json数据----------------------------------begin
                fc_time = self.getTime(_date, i)  # 第一列日期留空白

                txt_ele, json_ele = self.getfore(_date, i, methond, para1, para2)  # 获取
                del txt_ele
                for j in range(len(json_ele)):
                    if str(json_ele[j]).strip() == "--":
                        json_ele[j]=np.nan

                JsonData = self.getJsonData(fc_time, json_ele)  # dict()
                ## json
                json_data_list.append(JsonData)

                # 创建json数据----------------------------------end
                if i in self.m_hours:
                    osprint2('SUCCESS', f"hour in={str(i)}")
            # 创建json文件----------------------------------begin
            file_name = self._name + _date + ".json"
            file = os.path.join(file_path, file_name)
            if self._location is None:
                new_dict = {"location": {"lon": self.s_lon, "lat": self.s_lat}, "data": json_data_list}
            else:
                new_dict = {"location": self._location, "data": json_data_list}

            with open(file, "w") as f:
                json.dump(new_dict, f, ensure_ascii=False, indent=10)
            # 写入json文件内容----------------------------------end

    def getJsonData(self, fc_time, json_ele):
        postData = {'date': fc_time}
        for i in range(len(self.value_list)):
            postData[self.title_list[i]] = np.float64(json_ele[i])

        return postData
    
    def getTxtJsonData(self, fc_time, json_ele):
        postJsonData = {'date': fc_time}
        PostTxtData = [fc_time]
        for i in range(len(self.value_list)):
            postJsonData[self.title_list[i]] = json_ele[i]
            PostTxtData.append(str(json_ele[i]).format('.2f').rjust(14, " "))

        return postJsonData, PostTxtData

    def getfore(self, _date, _idx, methond, para1, para2):
        # sourcery skip: low-code-quality
        """
        获取数据
        :param methond:
        :param para2:
        :param para1:
        :param _date: 任务日期
        :param _idx: 小时
        :return: 有效波高，平均周期，谱峰周期，平均波向...
        """
        # 时间处理，将UTC转为CST
        date1 = datetime.datetime.strptime(f'{_date}00', '%Y%m%d%H')
        _idx = _idx - self.time_zone  # 向前8个钟头，找到CST当天的0点, 如CST应为 - 8
        date2 = date1 + datetime.timedelta(hours=_idx)
        idx = _idx % 24
        filename = os.path.join(self._input, f"{date2.strftime('%Y%m%d')}/ww3.{date2.strftime('%Y%m%d')}.nc")  # ww3 nc文件名

        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        interp_value=[]
        interp_num=[]
        if methond=='nearest':
            for ele in self.value_list:
                element= w.variables[ele][idx, :, :]
                if ele=='fp':
                    temp=1/self.gts_nearest(lons, lats, element, self.s_lon, self.s_lat)
                elif ele=='hmaxe':
                    element = w.variables['hs'][idx, :, :]
                    temp=1.46*self.gts_nearest(lons, lats, element, self.s_lon, self.s_lat)
                else:
                    temp=self.gts_nearest(lons, lats, element, self.s_lon, self.s_lat)
                interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)
        elif methond=='onelon':
            for ele in self.value_list:
                element= w.variables[ele][idx, :, :]
                if ele=='dir':
                    u = -1 * np.sin(np.deg2rad(element))  # 波向分解
                    u_new=self.gts_onelon(lats, u, para1, para2)
                    v = -1 * np.cos(np.deg2rad(element))  # 波向分解
                    v_new=self.gts_onelon(lats, v, para1, para2)
                    temp=np.rad2deg(np.arctan2(u_new, v_new))+180
                elif ele=='fp':
                    temp = 1/self.gts_onelon(lats, element, para1, para2)
                elif ele=='hmaxe':
                    element = w.variables['hs'][idx, :, :]
                    temp = 1.46*self.gts_onelon(lats, element, para1, para2)
                else:
                    temp=self.gts_onelon(lats, element, para1, para2)
                interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)
        elif methond=='onelat':
            for ele in self.value_list:
                element= w.variables[ele][idx, :, :]
                if ele == 'dir':
                    u = -1 * np.sin(np.deg2rad(element))  # 波向分解
                    u_new = self.gts_onelat(lats, u, para1, para2)
                    v = -1 * np.cos(np.deg2rad(element))  # 波向分解
                    v_new = self.gts_onelat(lats, v, para1, para2)
                    temp=np.rad2deg(np.arctan2(u_new, v_new))+180
                elif ele=='fp':
                    temp = 1/self.gts_onelat(lats, element, para1, para2)
                elif ele=='hmaxe':
                    element = w.variables['hs'][idx, :, :]
                    temp = 1.46*self.gts_onelat(lats, element, para1, para2)
                else:
                    temp = self.gts_onelat(lats, element, para1, para2)
                interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)

        elif methond=='linear':
            for ele in self.value_list:
                element= w.variables[ele][idx, :, :]
                if ele == 'dir':
                    u = -1 * np.sin(np.deg2rad(element))  # 波向分解
                    u_new = self.gts_linear(lons, lats, u, self.s_lon, self.s_lat, para1, para2)
                    v = -1 * np.cos(np.deg2rad(element))  # 波向分解
                    v_new = self.gts_linear(lons, lats, v, self.s_lon, self.s_lat, para1, para2)
                    temp=np.rad2deg(np.arctan2(u_new, v_new))+180
                elif ele=='fp':
                    temp = 1/self.gts_linear(lons, lats, element, self.s_lon, self.s_lat, para1, para2)
                elif ele=='hmaxe':
                    element = w.variables['hs'][idx, :, :]
                    temp =1.46* self.gts_linear(lons, lats, element, self.s_lon, self.s_lat, para1, para2)
                else:
                    temp=self.gts_linear(lons, lats, element, self.s_lon, self.s_lat, para1, para2)
                interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)
        elif methond=='idw':
            for ele in self.value_list:
                element= w.variables[ele][idx, :, :]
                if ele =='dir':
                    u = -1 * np.sin(np.deg2rad(element))  # 波向分解
                    u_new = self.gts_idw(u, para1, para2)
                    v = -1 * np.cos(np.deg2rad(element))  # 波向分解
                    v_new = self.gts_idw(v, para1, para2)
                    temp=np.rad2deg(np.arctan2(u_new, v_new))+180
                elif ele == 'fp':
                    temp = 1/self.gts_idw(element, para1, para2)
                elif ele=='hmaxe':
                    element = w.variables['hs'][idx, :, :]
                    temp=1.46*self.gts_idw(element, para1, para2)
                else:
                    temp=self.gts_idw(element, para1, para2)
                interp_value, interp_num = self.return_str_value(interp_value, interp_num, temp)
        return interp_value, interp_num

    @staticmethod
    def return_str_value(interp_value, interp_num, _value, retract=14):
        if str(_value).strip() == "--":
            _value = np.nan
        _value = np.round(_value, 2)
        interp_value.append(str(_value).format('.2f').rjust(retract, " "))
        interp_num.append(_value)
        return interp_value, interp_num

    @staticmethod
    def find_index(g_lon, g_lat, s_lon, s_lat):
        """
        获取经纬度索引
        :param g_lon: 一维格点经度
        :param g_lat: 一维格点纬度
        :param s_lon: 散点经度
        :param s_lat: 散点纬度
        :return: 纬度索引 经度索引
        """
        latli = np.argmin(np.abs(g_lat - s_lat))
        lonli = np.argmin(np.abs(g_lon - s_lon))
        return latli, lonli

    def search_tree(self, g_lon, g_lat, s_lon, s_lat, k):
        # sourcery skip: list-comprehension, merge-duplicate-blocks
        """
        找到离指定散点最近的K个格点
        :param g_lon: 一维格点经度
        :param g_lat: 一维格点纬度
        :param s_lon: 散点经度
        :param s_lat: 散点纬度
        :param k: 寻找近邻点的个数
        :return: 格点经纬度fp 索引id 距离d
        """
        lat_number = len(g_lat)
        lon_number = len(g_lon)
        # 生成格点坐标，放进一个二维平面
        grid = []
        for i, j in itertools.product(range(lon_number), range(lat_number)):
            x = [g_lon[i], g_lat[j]]
            grid.append(x)
        point = np.array(grid)

        # 用于快速查找的KDTree类
        kt = spt.KDTree(data=point, leafsize=10)
        ckt = spt.cKDTree(point)  # 用C写的查找类，执行速度更快
        find_point = np.array([s_lon, s_lat])  # 原点
        distance, sequence = kt.query(find_point, k)  # 返回最近邻点的距离d和在数组中的顺序sequence
        fp = []
        for i in range(len(point)):
            if k == 1:
                if i == sequence:
                    fp.append(point[i])
            elif i in sequence:
                fp.append(point[i])

        index = []
        for f in fp:
            id = self.find_index(g_lon, g_lat, f[0], f[1])
            index.append(id)

        # 找出最近邻点的位置坐标
        # for i in range(len(point)):
        #     if i == x:
        #         fp = point[i]

        # print('最近邻点距离：', d)
        # print('最近邻点位置：', fp)
        # print('最近邻点索引', index)
        return fp, index, distance

    @staticmethod
    def detect_nan(value):
        """
        判断格点数据是否满足双线性插值条件
        :param value: 取出的格点数据
        :return: 满足则返回1，不满足则返回0
        """
        try:
            if value.mask:
                return 0
        except Exception:
            return 1

    def gts_nearest(self, g_lon, g_lat, value, s_lon, s_lat):
        # sourcery skip: inline-immediately-returned-variable
        """
        最近插值
        :param g_lon: 一维格点经度
        :param g_lat: 一维格点纬度
        :param value: 二维数据
        :param s_lon: 散点经度
        :param s_lat: 散点纬度
        :return: 插值后的数据
        """
        latli, lonli = self.find_index(g_lon, g_lat, s_lon, s_lat)
        result = value[latli, lonli]
        return result

    @staticmethod
    def gts_linear(g_lon, g_lat, value, s_lon, s_lat, fp, id):
        # sourcery skip: inline-immediately-returned-variable
        """
        双线性插值
        :param g_lon: 一维格点经度
        :param g_lat: 一维格点纬度
        :param value: 二维数据
        :param s_lon: 散点经度
        :param s_lat: 散点纬度
        :param fp: 最近邻点的经纬度
        :param id: 最近邻点的索引
        :return: 插值后的数据
        """
        # 获取分辨率
        lat_re = g_lat[1] - g_lat[0]
        lon_re = g_lon[1] - g_lon[0]
        # 获取值
        bot_l = value[id[0]]  # 左下
        bot_r = value[id[1]]  # 右下
        top_l = value[id[2]]  # 左上
        top_r = value[id[3]]  # 右上
        # 获取经纬度
        lon_l = fp[0][0]
        lon_r = fp[2][0]
        lat_t = fp[1][1]
        lat_b = fp[0][1]

        # 第一次插值
        t_line = ((lon_r - s_lon) / lon_re) * top_l + ((s_lon - lon_l) / lon_re) * top_r
        b_line = ((lon_r - s_lon) / lon_re) * bot_l + ((s_lon - lon_l) / lon_re) * bot_r

        # 第二次插值
        result = ((lat_t - s_lat) / lat_re) * b_line + ((s_lat - lat_b) / lat_re) * t_line

        return result

    def gts_onelon(self, g_lat, value, fp, id):
        # sourcery skip: inline-immediately-returned-variable
        """
        只需要一次线性插值
        :param g_lat:
        :param value:
        :param fp:
        :param id:
        :return:
        """
        # 获取分辨率
        lat_re = g_lat[1] - g_lat[0]

        # 获取值
        top = value[id[0]]  # 下
        bot = value[id[1]]  # 上

        # 获取纬度
        lat_t = fp[1][1]
        lat_b = fp[0][1]

        # 插值
        result = ((lat_t - self.s_lat) / lat_re) * bot + ((self.s_lat - lat_b) / lat_re) * top
        return result

    def gts_onelat(self, g_lon, value, fp, id):
        # sourcery skip: inline-immediately-returned-variable
        # 获取分辨率
        lon_re = g_lon[1] - g_lon[0]

        # 获取值
        left = value[id[0]]  # 左
        right = value[id[1]]  # 右

        # 获取经度
        lon_l = fp[0][0]
        lon_r = fp[1][0]

        # 插值
        result = ((lon_r - self.s_lon) / lon_re) * left + ((self.s_lon - lon_l) / lon_re) * right
        return result

    @staticmethod
    def gts_idw(value, cal_id, cal_pa):
        result=0
        for i in range(len(cal_id)):
            result=result+cal_pa[i]*value[cal_id[i]]
        return result

    def judge_method(self, _date):
        # sourcery skip: extract-duplicate-method, inline-variable, low-code-quality, remove-unnecessary-else
        # 读取文件
        _date1=datetime.datetime.strptime(f'{_date}', '%Y%m%d')
        filename = os.path.join(self._input, f"{_date1.strftime('%Y%m%d')}/ww3.{_date1.strftime('%Y%m%d')}.nc")  # ww3 nc文件名
        osprint2('INFO', '正在判断插值方法')
        w = netCDF4.Dataset(filename)  # 读取ww3_Wp的nc文件，传入w中
        lons = w.variables['longitude'][:]
        lats = w.variables['latitude'][:]
        value = w.variables[self.value_list[0]][0, :, :]

        if self.method == 'linear':  # 手动选择插值方法
            fp, id, d = self.search_tree(lons, lats, self.s_lon, self.s_lat, 4)
            osprint2('INFO', '手动选择插值方法 -- 双线性插值')
            return self.method, fp, id
        elif self.method == 'idw':
            cal_id, cal_pa = self.idw_para(lons, lats, value, self.s_lon, self.s_lat)
            osprint2('INFO', '手动选择插值方法 -- 反距离加权插值')
            return self.method, cal_id, cal_pa
        elif self.method == 'nearest':
            fp, id, d = self.search_tree(lons, lats, self.s_lon, self.s_lat, 1)
            osprint2('INFO', '手动选择插值方法 -- 最近邻插值')
            return self.method, fp, id
        else:  # 自动判断
            # 判断该散点是否落在格点经纬线上，如果在那么只需要进行一次插值即可
            if self.s_lon in lons and self.s_lat not in lats:
                fp, id, d = self.search_tree(lons, lats, self.s_lon, self.s_lat, 2)
                top = value[id[0]]  # 下
                test = [self.detect_nan(top)]
                bot = value[id[1]]  # 上
                test.append(self.detect_nan(bot))
                if 0 < test.count(0) < 2:
                    osprint2('INFO', '该点附近存在nan值，不满足线性插值条件，使用反距离权重插值')
                    method = 'idw'
                    cal_id, cal_pa=self.idw_para(lons, lats, value, self.s_lon, self.s_lat)
                    return method, cal_id, cal_pa
                elif test.count(0) == 2:
                    osprint2('INFO', '该点附近全是nan值，识别为陆地，使用临近插值')
                    method = 'nearest'
                    return method, fp, id
                else:
                    osprint2('INFO', '该点落在格点经线上，使用一次线性插值即可')
                    method='onelon'
                    return method, fp, id
    
            elif self.s_lat in lats and self.s_lon not in lons:
                fp, id, d = self.search_tree(lons, lats, self.s_lon, self.s_lat, 2)
                left = value[id[0]]  # 左
                test = [self.detect_nan(left)]
                right = value[id[1]]  # 右
                test.append(self.detect_nan(right))
                if 0 < test.count(0) < 2:
                    osprint2('INFO', '该点附近存在nan值，不满足线性插值条件，使用反距离权重插值')
                    method = 'idw'
                    cal_id, cal_pa = self.idw_para(lons, lats, value, self.s_lon, self.s_lat)
                    return method, cal_id, cal_pa
    
                elif test.count(0) == 2:
                    osprint2('INFO', '该点附近全是nan值，识别为陆地，使用临近插值')
                    method = 'nearest'
                    return method, fp, id
                else:
                    osprint2('INFO', '该点落在格点经线上，使用一次线性插值即可')
                    method='onelat'
                    return method, fp, id
            else:
                fp, id, d = self.search_tree(lons, lats, self.s_lon, self.s_lat, 4)
                # print(fp)
                # print(id)
    
                # 判断该散点是否为格点
    
                if 0 in d:
                    osprint2('INFO', '该点为格点，使用临近插值')
                    method = 'nearest'
                    return method, fp, id
                else:
                    # 获取值
                    bot_l = value[id[0]]  # 左下
                    test = [self.detect_nan(bot_l)]
                    bot_r = value[id[1]]  # 右下
                    test.append(self.detect_nan(bot_r))
                    top_l = value[id[2]]  # 左上
                    test.append(self.detect_nan(top_l))
                    top_r = value[id[3]]  # 右上
                    test.append(self.detect_nan(top_r))
                    # print(bot_r, bot_l, top_r, top_l)
                    # 判断格点是否满足双线性插值条件
                    if 0 < test.count(0) < 4:
                        osprint2('INFO', '该点附近存在nan值，不满足线性插值条件，使用反距离权重插值')
                        method = 'idw'
                        cal_id, cal_pa = self.idw_para(lons, lats, value, self.s_lon, self.s_lat)
                        return method, cal_id, cal_pa
    
                    elif test.count(0) == 4:
                        osprint2('INFO', '该点附近全是nan值，识别为陆地，使用临近插值')
                        method = 'nearest'
                        return method, fp, id
                    else:
                        osprint2('INFO', '该点满足线性插值条件,使用双线性插值')
                        method = 'linear'
                        return method, fp, id

    def idw_para(self, g_lon, g_lat, value, s_lon, s_lat):
        lat_number = len(g_lat)
        lon_number = len(g_lon)
        # 生成格点坐标，放进一个二维平面
        grid = []
        for i, j in itertools.product(range(lon_number), range(lat_number)):
            x = [g_lon[i], g_lat[j]]
            grid.append(x)
        point = np.array(grid)

        # 用于快速查找的KDTree类
        kt = spt.KDTree(data=point, leafsize=10)
        ckt = spt.cKDTree(point)  # 用C写的查找类，执行速度更快
        find_point = np.array([s_lon, s_lat])  # 原点
        distance, sequence = kt.query(find_point, self.search_point)  # 返回最近邻点的距离d和在数组中的顺序sequence
        fp = []  # 存格点
        if self.search_point == 1:
            fp.extend(point[i] for i in range(len(point)) if i == sequence)
        else:
            fp.extend(point[p] for p in sequence)
        index = []  # 存索引
        for f in fp:
            id = self.find_index(g_lon, g_lat, f[0], f[1])
            index.append(id)

        cal_id=[]  # 存可以进入计算的点的索引
        cal_d=[]
        for i in range(len(index)):
            if self.detect_nan(value[index[i]]):
                cal_id.append(index[i])
                cal_d.append(distance[i])
        #反距离加权参数的计算
        cal_pa=[]
        total = 0
        for item in cal_d:
            total = total + 1 / item
        cal_pa.extend((1/dis)/total for dis in cal_d)
        return cal_id, cal_pa

    @staticmethod
    def uv2dir(u, v):
        # u,v为波的去向，该函数得到的角度为波的来向(单位为：度，正北为0°，顺时针旋转)
        u *= -1
        v *= -1
        degree = math.degrees(np.arctan(u / v))
        if u > 0 and v > 0:
            pass
        elif u > 0 > v:
            # 这种情况下原degree < 0
            degree += 180
        elif u < 0 < v:
            # 这种情况下原degree < 0
            degree += 360
        elif u < 0 and v < 0:
            degree += 180
        elif u == 0 and v > 0:
            degree = 0
        elif u == 0 and v < 0:
            degree = 180
        elif v == 0 and u > 0:
            degree = 90
        elif v == 0 and u < 0:
            degree = 270
        return degree


def convertToTime(strDate):
    """
    将 %Y%m%d 格式的8位字符串，转换为日期
    :param strDate: %Y%m%d 格式的8位日期字符串
    :return: datetime 类型的日期
    """
    date = datetime.datetime.now()  # 默认取当天日期
    
    with contextlib.suppress(Exception):
        if len(strDate) == 8:
            date = datetime.datetime.strptime(strDate, "%Y%m%d")
        elif len(strDate) == 10:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H")
        elif len(strDate) == 12:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M")
        elif len(strDate) == 14:
            date = datetime.datetime.strptime(strDate, "%Y%m%d%H%M%S")
    return date
    

def getToday():
    """
    返回当天日期的 %Y%m%d 格式的8位字符串
    :return:
    """
    return datetime.datetime.now().strftime('%Y%m%d')


## 外参部分 argparse
def get_parser():
    # Grid2scatter.conf文件路径
    baysalt_path = os.path.dirname(os.path.abspath(__file__))
    Conf_Grid2scatter_file = os.path.join(baysalt_path, 'Configures', 'Grid2scatter.conf')
    # 区分参数大小写
    parser = argparse.ArgumentParser(
        description=__doc__,
        epilog="End help",
        add_help=False,
        prog=f'./{os.path.basename(__file__)}',
        conflict_handler='error',
        formatter_class=argparse.RawTextHelpFormatter,
        allow_abbrev=True,
        prefix_chars='-+',
        # description='Usages',
    )
    parser.add_argument_group(
        'Usages Help', '%(prog)s \n'
        '                -m liner \n'
        '                -t 8 \n'
        '                -p 4 \n'
        '                -lon 114 \n'
        '                -lat 35 \n'
        '                -iv hs lm \n'
        '                -ov swh mwl \n'
        '                -i  /data/ForecastSystem/WW3_6.07/Output/ww3_WPacific/ \n'
        '                -o  ./ \n'
        '                -n try \n'
        '                -d 20230301 \n'
        '%(prog)s \n'
        '                -conf 1 \n'
        )
    parser.usage = '%(prog)s [options] parameter'
    parser.add_argument('-h',    '--help', action='help', help='show this help message and exit')
    parser.add_argument('-m',    required=False, type=str,   metavar='', default=None,                                                 help='set method',              choices=['linear', 'nearest', 'idw', 'auto'])
    parser.add_argument('-t',    required=False, type=int,   metavar='', default=8,                                                    help='set time zone')
    parser.add_argument('-p',    required=False, type=int,   metavar='', default=4,                                                    help='set search point')
    parser.add_argument('-lon',  required=False, type=float, metavar='', default=0,                                                    help='set longitude')
    parser.add_argument('-lat',  required=False, type=float, metavar='', default=0,                                                    help='set latitude')
    parser.add_argument('-iv',   required=False, type=str,   metavar='', default=['hs', 't02', 'dir', 'fp',   'lm',  'hmaxe'],         help='set input variable name', choices=['hs', 't02', 'dir', 'fp', 'lm', 'hmaxe'], nargs='+')
    parser.add_argument('-ov',   required=False, type=str,   metavar='', default=['swh', 'mwp', 'mwd', 'pp1d', 'mwl', 'hmax'],         help='set output variable name', nargs='+')
    parser.add_argument('-i',    required=False, type=str,   metavar='', default='/data/ForecastSystem/WW3_6.07/Output/ww3_WPacific/', help='set input path')
    parser.add_argument('-o',    required=False, type=str,   metavar='', default='./',                                                 help='set output path')
    parser.add_argument('-n',    required=False, type=str,   metavar='', default='g2s_',                                                help='set output file name')
    parser.add_argument('-d',    required=False, type=str,   metavar='', default=getToday(),                               help='set date')
    parser.add_argument('-conf', required=False, type=str,   metavar='', default=Conf_Grid2scatter_file,                               help='set conf')

    return parser.parse_args(), Conf_Grid2scatter_file


def print_args(_args):
    print('-----------------args default-----------------')
    for k, v in vars(_args).items():
        print(f'{k} : {v}')
    print('-----------------args default-----------------')


def example():
    args, _ = get_parser()
    print_args(args)

    X = Grid2Sca(122.247, 35.22,
                 _input=args.i,
                 _output=args.o,
                 _name=args.n,
                 value_list=args.iv,
                 title_list=args.ov,
                 _method=args.m)
    X.makefore(_date=args.d, _json=True, _txt=False)


if __name__ == '__main__':
    example()



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/9/18 15:37
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
from .xr_attrs import *
from .xr_wave import *



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/4/12 16:45
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Cprintf
#  版本 : python 3
#  摘要 :
"""

"""
from __future__ import annotations

import datetime
import logging
from logging.handlers import RotatingFileHandler, TimedRotatingFileHandler
from typing import TYPE_CHECKING, Any, Optional, cast

import colorlog

from .commonCode import rmfiles, whether_instanced

__all__ = ['Blog', 'test', 'debug', 'info', 'warning', 'error', 'critical', 'exception', 'log']

_DEFAULT_BLOG: Optional["Blog"] = None

if TYPE_CHECKING:
    class _BlogLogger(logging.Logger):
        def test(self, msg: Any, *args: Any, **kwargs: Any) -> None: ...
else:
    _BlogLogger = logging.Logger


def _get_default_blog() -> "Blog":
    global _DEFAULT_BLOG
    if _DEFAULT_BLOG is None:
        _DEFAULT_BLOG = Blog()
    return _DEFAULT_BLOG


class Blog:
    """
    先创建日志记录器（logging.getLogger），然后再设置日志级别（logger.setLevel），
    接着再创建日志文件，也就是日志保存的地方（logging.FileHandler），然后再设置日志格式（logging.Formatter），
    最后再将日志处理程序记录到记录器（addHandler）

    Loggers：记录器，提供应用程序代码能直接使用的接口；
    Handlers：处理器，将记录器产生的日志发送至目的地；
    Filters：过滤器，提供更好的粒度控制，决定哪些日志会被输出；
    Formatters：格式化器，设置日志内容的组成结构和消息字段。
            %(name)s Logger的名字         #也就是其中的.getLogger里的路径,或者我们用他的文件名看我们填什么
            %(levelno)s 数字形式的日志级别  #日志里面的打印的对象的级别
            %(levelname)s 文本形式的日志级别 #级别的名称
            %(pathname)s 调用日志输出函数的模块的完整路径名，可能没有
            %(filename)s 调用日志输出函数的模块的文件名
            %(module)s 调用日志输出函数的模块名
            %(funcName)s 调用日志输出函数的函数名
            %(lineno)d 调用日志输出函数的语句所在的代码行
            %(created)f 当前时间，用UNIX标准的表示时间的浮 点数表示
            %(relativeCreated)d 输出日志信息时的，自Logger创建以 来的毫秒数
            %(asctime)s 字符串形式的当前时间。默认格式是 “2003-07-08 16:49:45,896”。逗号后面的是毫秒
            %(thread)d 线程ID。可能没有
            %(threadName)s 线程名。可能没有
            %(process)d 进程ID。可能没有
            %(message)s用户输出的消息
    ==============================================================
    Updates:
        1. 2025-10-21 14:10:    Added self._logger.propagate=False to prevent duplicate log printing. by Christmas.
    """

    # ---- custom log level registration (one-liner per level) ----
    _CUSTOM_LEVELS_REGISTERED = set()

    @classmethod
    def register_level(cls, level: int, level_name: str, method_name: str | None = None):
        """
        注册自定义日志级别，并注入对应的 Logger 方法。
        示例:
            Blog.register_level(5, "TEST") -> logger.test(...)
        """
        if method_name is None:
            method_name = level_name.lower()

        key = (level, level_name, method_name)
        if key in cls._CUSTOM_LEVELS_REGISTERED:
            return

        # 1) 注册等级名称（用于 %(levelname)s 和 getLevelName）
        logging.addLevelName(level, level_name)

        # 2) 同步为 logging.LEVELNAME 常量（可选，方便外部使用）
        if not hasattr(logging, level_name):
            setattr(logging, level_name, level)

        # 3) 给 Logger 注入同名方法 logger.<method_name>(...)，用于以该等级输出日志
        if not hasattr(logging.Logger, method_name):
            def _log_for_level(self, msg, *args, **kwargs):
                if self.isEnabledFor(level):
                    # 确保自定义方法的调用位置正确（funcName/lineno 指向用户代码，而不是 _log_for_level）
                    kwargs.setdefault("stacklevel", 2)
                    self._log(level, msg, args, **kwargs)

            _log_for_level.__name__ = method_name
            setattr(logging.Logger, method_name, _log_for_level)

        cls._CUSTOM_LEVELS_REGISTERED.add(key)
    # ------------------------------------------------------------
    
    def __init__(self, _loger_name='Christmas',  # 日志记录器名称
                 _log_filename=None,  # 日志文件名
                 _switch_write_all_log=False,  # 是否写入日志
                 _switch_write_bug_log=False,  # 是否写入日志
                 _switch_print_log=True,  # 是否打印日志
                 _switch_write_test_log=False,  # 是否写入测试日志
                 _switch_write_debug_log=False,  # 是否写入单独的debug日志
                 _switch_write_info_log=False,  # 是否写入单独的info日志
                 _switch_write_warning_log=False,  # 是否写入单独的warning日志
                 _switch_write_error_log=False,  # 是否写入单独的error日志
                 _switch_write_critical_log=False,  # 是否写入单独的critical日志
                 _switch_bold_print=True,  # 是否打印彩色日志
                 **kwargs
                 ):

        self.CRITICAL = logging.CRITICAL
        self.FATAL = self.CRITICAL
        self.ERROR = logging.ERROR
        self.WARNING = logging.WARNING
        self.WARN = self.WARNING
        self.INFO = logging.INFO
        self.DEBUG = logging.DEBUG
        self.TEST = 5  # 自定义日志级别 TEST：开发期低噪声调试等级（低于 DEBUG）

        # Register custom levels (process-wide)
        self.register_level(self.TEST, "TEST")
        self.NOTSET = logging.NOTSET

        self.loger_name = _loger_name  # 日志记录器名称
        self.logfile_name = _log_filename  # 日志文件名
        self.switch_write_all_log = _switch_write_all_log  # 是否写入全部日志
        self.switch_write_error_plus_log = _switch_write_bug_log  # 是否写入错误日志
        self.switch_print_log = _switch_print_log  # 是否打印日志
        self.switch_write_test_log = _switch_write_test_log  # 是否写入测试日志
        self.switch_write_debug_log = _switch_write_debug_log  # 是否写入单独的debug日志
        self.switch_write_info_log = _switch_write_info_log  # 是否写入单独的info日志
        self.switch_write_warning_log = _switch_write_warning_log  # 是否写入单独的warning日志
        self.switch_write_error_log = _switch_write_error_log  # 是否写入单独的error日志
        self.switch_write_critical_log = _switch_write_critical_log  # 是否写入单独的critical日志
        self.switch_bold_print = _switch_bold_print  # 是否打印彩色日志
        
        self.log_level = None  # 日志级别
        self.maxBytes = None  # 日志文件大小
        self.backupCount = None  # 日志文件数量
        self.write_mode = None  # 日志写入模式
        self.colors_config = None  # 控制台打印颜色配置信息
        self.log_ddt_fmt = None  # 日志文件名时间格式
        self.all_log_filename = None  # 所有日志文件路径
        self.bug_log_filename = None  # 错误日志文件路径
        self.test_log_filename = None  # 测试日志文件路径
        self.debug_log_filename = None  # debug日志文件路径
        self.info_log_filename = None  # info日志文件路径
        self.warning_log_filename = None  # warning日志文件路径
        self.error_log_filename = None  # error日志文件路径
        self.critical_log_filename = None  # critical日志文件路径
        self.Rotating = None  # 日志文件处理器 time or size
        self.when = None  # 日志文件处理器 time --> S M H D W
        self.interval = None  # 日志文件处理器 time --> 间隔
        self._logger = cast(_BlogLogger, logging.getLogger(self.loger_name))  # 创建日志记录器
        self._logger.propagate = False  # 防止日志重复打印 不向上冒泡
        self._console_configured = False  # 控制台/文件 handler 是否已配置完成（避免重复创建）
        
        self.PARA_DEFAULT = {
            'propagate': False,  # 防止日志重复打印 不向上冒泡
            'maxBytes': 1024 * 1024 * 10,  # 日志文件大小
            'backupCount': 5,  # 日志文件数量
            'colors_config': self.__set_console_color_default(),  # 日志输出颜色
            'log_level': logging.INFO,  # 日志级别
            'log_ddt_fmt': '%Y-%m-%d',  # 日志文件名时间格式
            'logfile_name': 'log',  # 日志文件名
            'write_mode': 'a',  # 日志文件写入模式
            'all_log_filename': 'ALL.log',  # 日志文件路径
            'bug_log_filename': 'BUG.log',  # 日志文件路径
            'test_log_filename': 'test.log',  # 日志文件路径
            'debug_log_filename': 'debug.log',  # 日志文件路径
            'info_log_filename': 'info.log',  # 日志文件路径
            'warning_log_filename': 'warning.log',  # 日志文件路径
            'error_log_filename': 'error.log',  # 日志文件路径
            'critical_log_filename': 'critical.log',  # 日志文件路径
            'Rotating': 'size',  # 日志文件处理器 time or size
            'when': 'D',  # 日志文件处理器 time --> S M H D W
            'interval': 1,  # 日志文件处理器 time --> 间隔
        }
        if self.logfile_name is not None:
            self.PARA_DEFAULT['all_log_filename'] = f'{self.logfile_name}_ALL.log'
            self.PARA_DEFAULT['bug_log_filename'] = f'{self.logfile_name}_BUG.log'
            self.PARA_DEFAULT['test_log_filename'] = f'{self.logfile_name}_test.log'
            self.PARA_DEFAULT['debug_log_filename'] = f'{self.logfile_name}_debug.log'
            self.PARA_DEFAULT['info_log_filename'] = f'{self.logfile_name}_info.log'
            self.PARA_DEFAULT['warning_log_filename'] = f'{self.logfile_name}_warning.log'
            self.PARA_DEFAULT['error_log_filename'] = f'{self.logfile_name}_error.log'
            self.PARA_DEFAULT['critical_log_filename'] = f'{self.logfile_name}_critical.log'
        
        self.PARA = self.PARA_DEFAULT
        self.setup_Blog(**kwargs)
        # self.console()
    
    def setup_Blog(self, **kwargs):
        
        self.PARA.update(kwargs)
        self._logger.propagate = self.PARA['propagate']  # 防止日志重复打印 不向上冒泡
        self.maxBytes = self.PARA['maxBytes']
        self.backupCount = self.PARA['backupCount']
        self.colors_config = self.PARA['colors_config']
        self.log_level = self.PARA['log_level']
        self.log_ddt_fmt = self.PARA['log_ddt_fmt']
        self.write_mode = self.PARA['write_mode']
        self.all_log_filename = self.PARA['all_log_filename']
        self.bug_log_filename = self.PARA['bug_log_filename']
        self.test_log_filename = self.PARA['test_log_filename']
        self.debug_log_filename = self.PARA['debug_log_filename']
        self.info_log_filename = self.PARA['info_log_filename']
        self.warning_log_filename = self.PARA['warning_log_filename']
        self.error_log_filename = self.PARA['error_log_filename']
        self.critical_log_filename = self.PARA['critical_log_filename']
        self.Rotating = self.PARA['Rotating']
        self.when = self.PARA['when']
        self.interval = self.PARA['interval']

        # 看看还有没有没用的kwargs键值对，有的话报错提示
        for key in kwargs.keys():
            if key not in self.PARA_DEFAULT.keys():
                raise KeyError(f'未知的参数: {key}')
        
        if self.write_mode == 'w':  # 删除已存在的日志文件
            rmfiles(self.all_log_filename,
                    self.bug_log_filename,
                    self.test_log_filename,
                    self.debug_log_filename,
                    self.info_log_filename,
                    self.warning_log_filename,
                    self.error_log_filename,
                    self.critical_log_filename,
                    )
        self.__rm_single_FileHandler()  # 删除日志记录器handler
        self._console_configured = False  # 参数更新后需要重新构造 handler
    
    def __init_logger_handler(self, logfile_path, Rotating='time', when='H', interval=1):
        """
        创建日志记录器handler，用于收集日志
        :param logfile_path: 日志文件路径
        :return: 日志记录器
        """
        # 写入文件，如果文件超过1M大小时，切割日志文件
        if Rotating == 'size':
            handler = RotatingFileHandler(filename=logfile_path, maxBytes=self.maxBytes, encoding='utf-8',
                                          mode=self.write_mode, backupCount=self.backupCount)
        elif Rotating == 'time':
            try:
                # noinspection PyArgumentList
                handler = TimedRotatingFileHandler(filename=logfile_path, when=when, interval=interval,
                                                   backupCount=self.backupCount, atTime=datetime.time(0, 0, 0),
                                                   delay=True, utc=False, mode=self.write_mode)
            except TypeError:
                handler = TimedRotatingFileHandler(filename=logfile_path, when=when, interval=interval,
                                                   backupCount=self.backupCount, atTime=datetime.time(0, 0, 0),
                                                   delay=True, utc=False)
        else:
            raise ValueError('Rotating参数错误')
        
        return handler
    
    def __set_log_handler(self, logger_handler, level=logging.DEBUG):
        """
        设置handler级别并添加到logger收集器
        :param logger_handler: 日志记录器
        :param level: 日志记录器级别
        """
        logger_handler.setLevel(level=level)
        self._logger.addHandler(logger_handler)  # 添加到logger收集器
    
    def __set_log_Filter(self, logger_handler, _level):
        """
        设置日志过滤器
        """
        if _level == self.TEST:
            ONLY_LOG = [self.TEST, logging.DEBUG]
        elif _level == logging.DEBUG:
            ONLY_LOG = [logging.DEBUG, logging.INFO]
        elif _level == logging.INFO:
            ONLY_LOG = [logging.INFO, logging.WARNING]
        elif _level == logging.WARNING:
            ONLY_LOG = [logging.WARNING, logging.ERROR]
        elif _level == logging.ERROR:
            ONLY_LOG = [logging.ERROR, logging.CRITICAL]
        elif _level == logging.CRITICAL:
            ONLY_LOG = [logging.CRITICAL, float('inf')]
            
        else:
            raise ValueError('日志级别错误')
        log_filter = logging.Filter()
        logger_handler.setLevel(ONLY_LOG[0])
        log_filter.filter = lambda record: (record.levelno < ONLY_LOG[1])  # 设置过滤等级
        logger_handler.addFilter(log_filter)
        self._logger.addHandler(logger_handler)  # 添加到logger收集器
    
    @staticmethod
    def __set_console_color_default():
        log_color = {
            # 颜色支持 blue蓝，green绿色，red红色，yellow黄色，cyan青色, purple紫色, white白色, black黑色, grey灰色, bold黑体
            'TEST': 'green',
            'DEBUG': 'cyan',
            'INFO': 'blue',
            'WARNING': 'yellow',
            'ERROR': 'red',
            'CRITICAL': 'purple',
            'EXCEPTION': 'red',
        }
        return log_color

    def __set_color_formatter(self, console_handle, colors_config):
        """
        设置输出格式-控制台
        :param console_handle: 终端日志记录器
        :param colors_config: 控制台打印颜色配置信息
        :return:
        """
        if self.switch_bold_print:
            formatter = colorlog.ColoredFormatter(
                # 输出那些信息，时间，文件名，函数名等等
                fmt='%(asctime)s ---> %(log_color)s\033[1m[%(levelname)s]\033[0m %(reset)s%(log_color)s%(message)s %(reset)s',
                # 时间格式
                datefmt='%Y-%m-%d %H:%M:%S',
                log_colors=colors_config,
                force_color=True,
            )
        else:
            formatter = colorlog.ColoredFormatter(
                # 输出那些信息，时间，文件名，函数名等等
                fmt='%(asctime)s ---> %(log_color)s%(levelname)s] %(reset)s%(log_color)s%(message)s %(reset)s',
                # 时间格式
                datefmt='%Y-%m-%d %H:%M:%S',
                log_colors=colors_config,
                force_color=True,
            )
        console_handle.setFormatter(formatter)
    
    @staticmethod
    def __set_log_formatter(file_handler):
        """
        设置日志输出格式-日志文件
        :param file_handler: 日志记录器
        """
        formatter = logging.Formatter(
            fmt='%(asctime)s ---> [%(levelname)s] %(filename)s -> %(funcName)s line:%(lineno)d : %(message)s',
            datefmt='%Y-%m-%d  %H:%M:%S'
        )
        file_handler.setFormatter(formatter)
    
    @staticmethod
    def __close_log_handler(logger_handler):
        """
        关闭日志记录器
        :param logger_handler: 日志记录器
        """
        logger_handler.close()
    
    def console(self):
        """
        构造日志收集器（用于管理控制台和文件日志输出的 handler）
        如果参数未变化且已构造过，就直接返回，避免重复创建 handler。
        """
        if getattr(self, "_console_configured", False):
            return self._logger

        # logger 的基础等级应严格遵循用户设置的 log_level；
        # 低于 log_level 的日志应在进入 handler 前被丢弃（这是 logging 的正常语义）
        self._logger.setLevel(self.log_level)

        if self._logger.hasHandlers():  # 如果已经有日志记录器，先关闭
            self._logger.handlers.clear()
        if self._logger.filters:  # 如果已经有日志过滤器，先关闭
            self._logger.filters.clear()
        if self.switch_write_all_log:  # 是否写入全部日志
            self.__write_all_log()
        if self.switch_write_error_plus_log:  # 是否写入错误日志
            self.__write_error_plus_log()
        if self.switch_print_log:  # 是否打印日志
            self.__print_log()
        if self.switch_write_test_log:  # 是否写入test日志
            self.__write_single_log(level=self.TEST, _level_log_name=self.test_log_filename)
        if self.switch_write_debug_log:  # 是否写入debug日志
            self.__write_single_log(level=logging.DEBUG, _level_log_name=self.debug_log_filename)
        if self.switch_write_info_log:  # 是否写入info日志
            self.__write_single_log(level=logging.INFO, _level_log_name=self.info_log_filename)
        if self.switch_write_warning_log:  # 是否写入warning日志
            self.__write_single_log(level=logging.WARNING, _level_log_name=self.warning_log_filename)
        if self.switch_write_error_log:  # 是否写入error日志
            self.__write_single_log(level=logging.ERROR, _level_log_name=self.error_log_filename)
        if self.switch_write_critical_log:  # 是否写入critical日志
            self.__write_single_log(level=logging.CRITICAL, _level_log_name=self.critical_log_filename)

        self._console_configured = True
        return self._logger
    
    def __write_all_log(self):
        all_logger_handler = self.__init_logger_handler(self.all_log_filename, Rotating=self.Rotating, when=self.when,
                                                        interval=self.interval)  # 收集所有日志文件
        self.__set_log_formatter(all_logger_handler)  # 设置日志输出格式-all日志文件
        self.__set_log_handler(all_logger_handler, level=logging.DEBUG)  # 设置handler级别并添加到logger收集器
        self.__close_log_handler(all_logger_handler)  # 关闭handler
    
    def __write_error_plus_log(self):
        error_logger_handler = self.__init_logger_handler(self.bug_log_filename, Rotating=self.Rotating,
                                                          when=self.when, interval=self.interval)  # 收集错误日志信息文件
        self.__set_log_formatter(error_logger_handler)  # 设置日志输出格式-error日志文件
        self.__set_log_handler(error_logger_handler, level=logging.ERROR)  # 设置handler级别并添加到logger收集器
        self.__close_log_handler(error_logger_handler)  # 关闭handler
    
    def __print_log(self):
        console_handle = colorlog.StreamHandler()  # 创建终端日志记录器handler，用于输出到控制台
        self.__set_color_formatter(console_handle, self.colors_config)  # 设置输出格式-控制台
        self.__set_log_handler(console_handle, level=self.log_level)  # 设置handler级别并添加到终端logger收集器
        self.__close_log_handler(console_handle)  # 关闭handler
    
    def __write_single_log(self, level=logging.ERROR, _level_log_name='all.log'):  # 日志单独输出
        single_logger_handler = self.__init_logger_handler(_level_log_name, Rotating=self.Rotating, when=self.when,
                                                           interval=self.interval)  # 收集随机日志信息文件
        self.__set_log_formatter(single_logger_handler)  # 设置日志输出格式-single日志文件
        self.__set_log_Filter(single_logger_handler, level)  # 设置过滤器
        self.__close_log_handler(single_logger_handler)  # 关闭handler

    def __rm_single_FileHandler(self):  # 删除日志文件并关闭日志记录器
        handlers = self._logger.handlers
        for handler in handlers.copy():
            if isinstance(handler, logging.FileHandler):  # 找到文件处理器，进行删除操作
                self._logger.removeHandler(handler)
                handler.close()  # 关闭文件处理器，释放资源

    @property
    def logger(self) -> _BlogLogger:
        # 首次访问时才构造 handler；之后直接返回，避免重复创建
        self.console()
        return self._logger

    @staticmethod
    def addLevelName(level, levelName):
        logging.addLevelName(level, levelName)
    
    @staticmethod
    def getLevelName(level):
        return logging.getLevelName(level)

    def __getitem__(self, item):
        return getattr(self, item)

    def __del__(self):
        # self._logger.handlers.clear()
        # self._logger.filters.clear()
        # del self._logger
        pass

    def __str__(self):
        return 'Blog()'

    def __call__(self, *args, **kwargs):
        return self._logger


def test(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.log(blog.TEST, msg, *args, **kwargs)


def debug(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.debug(msg, *args, **kwargs)


def info(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.info(msg, *args, **kwargs)


def warning(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.warning(msg, *args, **kwargs)


def error(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.error(msg, *args, **kwargs)


def critical(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.critical(msg, *args, **kwargs)


def exception(msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.exception(msg, *args, **kwargs)


def log(level, msg, *args, **kwargs):
    blog = _get_default_blog()
    blog.logger.log(level, msg, *args, **kwargs)


def example_Blog():
    log2 = Blog(_loger_name='Christmas',  # 日志收集器名称
                _log_filename='ChristmasWRITING',  # 日志文件名前缀
                _switch_write_all_log=True,  # 是否写入全部日志 ALL.log
                _switch_write_bug_log=True,  # 是否写入错误日志 BUG.log
                _switch_print_log=True,  # 是否打印日志到控制台
                _switch_write_debug_log=True,  # 是否写入debug日志
                _switch_write_info_log=True,  # 是否写入info日志
                _switch_write_warning_log=True,  # 是否写入warning日志
                _switch_write_error_log=True,  # 是否写入error日志
                _switch_write_critical_log=True,  # 是否写入critical日志
                )
    log2.addLevelName(15, "CUSTOM")  # 自定义日志级别 15 --> CUSTOM
    log2.setup_Blog(write_mode='w')  # 日志写入模式
    log2.setup_Blog(maxBytes=1024 * 1024 * 5)  # 日志文件大小
    log2.setup_Blog(backupCount=5)  # 日志文件数量
    log2.setup_Blog(log_level=1)  # 日志级别
    log2.setup_Blog(propagate=False)  # 防止日志重复打印 不向上冒泡
    log2.setup_Blog(log_ddt_fmt='%Y-%m-%d %H:%M:%S')  # 日志时间格式
    log2.setup_Blog(colors_config={
        "CUSTOM": "yellow",
        'DEBUG': 'cyan',
        'INFO': 'blue',
        'WARNING': 'yellow',
        'ERROR': 'red',
        'CRITICAL': 'purple',
        'EXCEPTION': 'red'})  # 日志颜色配置
    log2.setup_Blog(Rotating='time')  # 日志切割模式
    log2.setup_Blog(when='D')  # 日志切割时间
    log2.setup_Blog(interval=1)  # 日志切割间隔
    x = log2.console()  # 日志收集器
    x.debug("这是debug信息")  # 日志输出
    x.log(15, "CUSTOM")  # 自定义日志级别
    x.info("这是日志信息")  # 日志输出
    x.warning("这是警告信息")  # 日志输出
    x.error("这是错误日志信息")  # 日志输出
    x.critical("这是严重级别信息")  # 日志输出


def example_Blog_simple():
    log_1 = Blog()
    log1 = log_1.console()
    log1.debug("这是debug信息")
    log1.info("这是日志信息")
    log1.warning("这是警告信息")
    log1.error("这是错误日志信息")
    log1.critical("这是严重级别信息")


if __name__ == '__main__':
    # example_Blog_simple()
    example_Blog()



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2024/11/6 16:36
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""
 
"""
import math
import warnings

import numpy as np

warnings.warn('This module has not been checked yet', UserWarning)


def haversine(origin, destination, units='km', ellipsoid="WGS84"):
    """
    To calculate distance between two points in Earth giving their coordinates (lat,lon)
    Parameters
    ----------
    origin:array like (lat,lon)
    coordinates of origin point
   
    destination: array like (lat,lon)
    coordinates of destinations points
   
    units: str
    units to return distance
    aviable units are kilometers (km), meters (m) and miles (mi)
   
    ellipsoid: String: type of projection
    aviables: Airy (1830), Bessel,Clarke (1880),FAI sphere,GRS-67,International,Krasovsky,NAD27,WGS66,WGS72,IERS (2003),WGS84- default WGS84
    return distance between points
    """
    if units == "km" or units == "kilometers":
        factor = 1
    elif units == "m" or units == "meters":
        factor = 1000
    elif units == "miles" or units == "mi":
        factor = 0.621371
    else:
        raise ValueError('aviable units are kilometers (km), meters (m) and miles (mi)')
    lat0, lon0 = origin
    
    ellipsoids = {
        "Airy (1830)": (6377.563, 6356.257),  # Ordnance Survey default
        "Bessel": (6377.397, 6356.079),
        "Clarke (1880)": (6378.249145, 6356.51486955),
        "FAI sphere": (6371, 6371),  # Idealised
        "GRS-67": (6378.160, 6356.775),
        "International": (6378.388, 6356.912),
        "Krasovsky": (6378.245, 6356.863),
        "NAD27": (6378.206, 6356.584),
        "WGS66": (6378.145, 6356.758),
        "WGS72": (6378.135, 6356.751),
        "WGS84": (6378.1370, 6356.7523),  # GPS default
        "IERS (2003)": (6378.1366, 6356.7519),
    }
    
    r1, r2 = ellipsoids[ellipsoid]
    lat, lon = destination
    mean_latitude = (lat0 + lat) / 2
    A = (r1 * r1 * math.cos(mean_latitude)) ** 2
    B = (r2 * r2 * math.sin(mean_latitude)) ** 2
    C = (r1 * math.cos(mean_latitude)) ** 2
    D = (r2 * math.sin(mean_latitude)) ** 2
    radius = np.sqrt((A + B) / (C + D))  # radius of the earth in km
    
    dlat = math.radians(lat - lat0)
    dlon = math.radians(lon - lon0)
    a = math.sin(dlat / 2) * math.sin(dlat / 2) + math.cos(math.radians(lat0)) \
        * math.cos(math.radians(lat)) * math.sin(dlon / 2) * math.sin(dlon / 2)
    c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))
    distance = radius * c * factor
    
    return distance


if __name__ == '__main__':
    lat1 = 22.5
    lon1 = -74.3
    lat2 = 23.8
    lon2 = -83.2
    
    haversine((lat1, lon1), (lat2, lon2), units='km')



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/9/18 15:38
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
import numpy as np

from .. import new_filename

__all__ = ['add_coors_attrs', 'add_lon_attrs', 'add_lat_attrs', 'add_time_attrs', 'add_TIME_attrs',
           'add_wind_attrs', 'add_temperature_attrs', 'add_precipitation_attrs', 'add_slp_attrs',
           'add_wave_attrs', 'add_ice_attrs',
           'fixed_encoding']


def add_coors_attrs(ds):
    """
    给xarray.Dataset的坐标添加属性
    :param ds: xarray.Dataset
    :return: xarray.Dataset
    """
    lon_name, lat_name, time_name, TIME_name = 'longitude', 'latitude', 'time', 'TIME'
    for i in ds.sizes.keys():  # ds.dims.keys(): --> FutureWarning
        lon_name = i if i in ['lon', 'longitude'] else 'longitude'
        lat_name = i if i in ['lat', 'latitude'] else 'latitude'
        time_name = i if i in ['time'] else 'time'
        TIME_name = i if i in ['TIME'] else 'TIME'
    add_lon_attrs(ds, lon_name)
    add_lat_attrs(ds, lat_name)
    add_time_attrs(ds, time_name)
    add_TIME_attrs(ds, TIME_name, dim_name='DateStr')
    return ds
    

def add_lon_attrs(_ds, lon_name):
    """
    给经度添加属性
    :param _ds: xarray.Dataset
    :param lon_name: 经度维度名
    :return: xarray.Dataset
    """
    _ds[lon_name].attrs = {
        'units': 'degrees_east',
        'long_name': 'longitude',
        'standard_name': 'longitude',
        'axis': 'X',
        'westernmost': np.round(_ds[lon_name].data.min(), 2),
        'easternmost': np.round(_ds[lon_name].data.max(), 2),
    }
    _ds[lon_name].encoding = {'_FillValue': None}
    return _ds


def add_lat_attrs(_ds, lat_name):
    """
    给纬度添加属性
    :param _ds: xarray.Dataset
    :param lat_name: 纬度维度名
    :return: xarray.Dataset
    """
    _ds[lat_name].attrs = {
        'units': 'degrees_north',
        'long_name': 'latitude',
        'standard_name': 'latitude',
        'axis': 'Y',
        'southernmost': np.round(_ds[lat_name].data.min(), 2),
        'northernmost': np.round(_ds[lat_name].data.max(), 2)
    }
    _ds[lat_name].encoding = {'_FillValue': None}
    return _ds


def add_time_attrs(_ds, time_name):
    """
    给时间添加属性
    :param _ds: xarray.Dataset
    :param time_name: 时间维度名
    :return: xarray.Dataset
    """
    _ds[time_name].attrs = {
        'units': 'seconds since 1970-01-01 00:00:00',
        'long_name': 'UTC time',
        'standard_name': 'UTC_time',
        'axis': 'T',
        'calendar': 'gregorian',
    }
    _ds[time_name].encoding = {'_FillValue': None, 'dtype': 'float64', 'unlimited_dims': 'time'}
    return _ds


def add_TIME_attrs(_ds, TIME_name, dim_name='DateStr'):
    """
    给TIME添加属性
    :param _ds: xarray.Dataset
    :param TIME_name: TIME名
    :param dim_name: TIME维度名
    :return: xarray.Dataset
    """
    _ds[TIME_name].attrs = {
        'reference_time': ''.join(_ds[TIME_name].data[0].astype(str))[:10],
        'long_name': 'UTC time',
        'standard_name': 'UTC_time',
        'calendar': 'gregorian',
        'start_time': ''.join(_ds[TIME_name].data[0].astype(str)),
        'end_time': ''.join(_ds[TIME_name].data[-1].astype(str))
    }
    _ds[TIME_name].encoding['char_dim_name'] = dim_name
    return _ds


def add_wind_attrs(_ds):
    """
    给xarray.Dataset的风添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    U10_name, V10_name = 'U10', 'V10'
    for i in list(_ds.data_vars.keys()):
        U10_name = i if i in ['U10', 'u10'] else 'U10'
        V10_name = i if i in ['V10', 'v10'] else 'V10'
    _ds[U10_name].attrs = {
        'units': 'm s-1',
        'long_name': '10 meter U wind component',
        'standard_name': 'eastward_wind',
    }
    _ds[V10_name].attrs = {
        'units': 'm s-1',
        'long_name': '10 meter V wind component',
        'standard_name': 'northward_wind',
    }
    _ds[U10_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)
    _ds[V10_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)
    return _ds


def add_temperature_attrs(_ds):
    """
    给xarray.Dataset的温度添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    T2_name = 'temperature'
    for i in list(_ds.data_vars.keys()):
        T2_name = i if i in ['T2', 't2'] else T2_name
    _ds[T2_name].attrs = {
        'units': 'K',
        'long_name': 'temperature at 2m',
        'standard_name': 'temperature_at_2m'
    }
    _ds[T2_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)
    return _ds


def add_precipitation_attrs(_ds):
    """
    给xarray.Dataset的降水添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    precipitation_name = 'precipitation'
    for i in list(_ds.data_vars.keys()):
        precipitation_name = i if i in ['tp', 'TP'] else precipitation_name
    _ds[precipitation_name].attrs = {
        'units': 'mm',
        'long_name': 'precipitation, positive for ocean gaining water',
        'standard_name': 'precipitation_positive_ocean_gaining_water'
    }
    _ds[precipitation_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)
    return _ds


def add_slp_attrs(_ds):
    """
    给xarray.Dataset的海平面气压添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    slp_name = 'slp'
    for i in list(_ds.data_vars.keys()):
        slp_name = i if i in ['slp', 'SLP'] else slp_name
    _ds[slp_name].attrs = {
        'units': 'Pa',
        'long_name': 'sea level pressure',
        'standard_name': 'sea_level_pressure'
    }
    _ds[slp_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)
    return _ds


def add_wave_attrs(_ds):
    """
    给xarray.Dataset的海浪添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    swh_name, mwp_name, mwd_name, mwl_name = 'swh', 'mwp', 'mwd', 'mwl'
    pp1d_name = 'pp1d'
    shww_name, shts_name, mdww_name, mdts_name = 'shww', 'shts', 'mdww', 'mdts'
    mpww_name, mpts_name, hmax_name = 'mpww', 'mpts', 'hmax'

    for i in list(_ds.data_vars.keys()):
        swh_name = i if i in ['swh'] else swh_name
        mwp_name = i if i in ['mwp'] else mwp_name
        mwd_name = i if i in ['mwd'] else mwd_name
        mwl_name = i if i in ['mwl'] else mwl_name
        pp1d_name = i if i in ['pp1d'] else pp1d_name
        shww_name = i if i in ['shww'] else shww_name
        shts_name = i if i in ['shts'] else shts_name
        mdww_name = i if i in ['mdww'] else mdww_name
        mdts_name = i if i in ['mdts'] else mdts_name
        mpww_name = i if i in ['mpww'] else mpww_name
        mpts_name = i if i in ['mpts'] else mpts_name
        hmax_name = i if i in ['hmax'] else hmax_name

    if swh_name in _ds:
        _ds[swh_name].attrs = {
            'units': 'm',
            'long_name': 'significant wave height',
            'standard_name': 'significant_wave_height'
        }
        _ds[swh_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mwp_name in _ds:
        _ds[mwp_name].attrs = {
            'units': 's',
            'long_name': 'mean wave period',
            'standard_name': 'mean_wave_period'
        }
        _ds[mwp_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mwd_name in _ds:
        _ds[mwd_name].attrs = {
            'units': 'degrees',
            'long_name': 'mean wave direction',
            'standard_name': 'mean_wave_direction',
            'direction_0': 'coming from the north',
            'direction_90': 'coming from the east'
        }
        _ds[mwd_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mwl_name in _ds:
        _ds[mwl_name].attrs = {
            'units': 'm',
            'long_name': 'mean wave length',
            'standard_name': 'mean_wave_length'
        }
        _ds[mwl_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if pp1d_name in _ds:
        _ds[pp1d_name].attrs = {
            'units': 's',
            'long_name': 'peak period',
            'standard_name': 'peak_period'
        }
        _ds[pp1d_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if shww_name in _ds:
        _ds[shww_name].attrs = {
            'units': 'm',
            'long_name': 'significant height of wind waves',
            'standard_name': 'significant_height_of_wind_waves'
        }
        _ds[shww_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if shts_name in _ds:
        _ds[shts_name].attrs = {
            'units': 'm',
            'long_name': 'significant height of total swell',
            'standard_name': 'significant_height_of_total_swell'
        }
        _ds[shts_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mdww_name in _ds:
        _ds[mdww_name].attrs = {
            'units': 'degrees',
            'long_name': 'mean direction of wind waves',
            'standard_name': 'mean_direction_of_wind_waves',
            'direction_0': 'coming from the north',
            'direction_90': 'coming from the east'
        }
        _ds[mdww_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mdts_name in _ds:
        _ds[mdts_name].attrs = {
            'units': 'degrees',
            'long_name': 'mean direction of total swell',
            'standard_name': 'mean_direction_of_total_swell',
            'direction_0': 'coming from the north',
            'direction_90': 'coming from the east'
        }
        _ds[mdts_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mpww_name in _ds:
        _ds[mpww_name].attrs = {
            'units': 's',
            'long_name': 'mean period of wind waves',
            'standard_name': 'mean_period_of_wind_waves'
        }
        _ds[mpww_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if mpts_name in _ds:
        _ds[mpts_name].attrs = {
            'units': 's',
            'long_name': 'mean period of total swell',
            'standard_name': 'mean_period_of_total_swell'
        }
        _ds[mpts_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if hmax_name in _ds:
        _ds[hmax_name].attrs = {
            'units': 'm',
            'long_name': 'maximum wave height',
            'standard_name': 'maximum_wave_height'
        }
        _ds[hmax_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    return _ds


def add_ice_attrs(_ds):
    """
    给xarray.Dataset的海冰添加属性
    :param _ds: xarray.Dataset
    :return: xarray.Dataset
    """
    aice_name = 'aice'
    tice_name = 'tice'

    for i in list(_ds.data_vars.keys()):
        aice_name = i if i in ['aice'] else aice_name
        tice_name = i if i in ['tice'] else tice_name

    if aice_name in _ds:
        _ds[aice_name].attrs = {
            'units': 'fraction',
            'long_name': 'sea ice concentration',
            'standard_name': 'sea_ice_concentration'
        }
        _ds[aice_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    if tice_name in _ds:
        _ds[tice_name].attrs = {
            'units': 'm',
            'long_name': 'sea ice thickness',
            'standard_name': 'sea_ice_thickness'
        }
        _ds[tice_name].encoding = dict(_FillValue=9.962100e+36, dtype='single', zlib=True)

    return _ds


def fixed_encoding(_ds, **kwargs):
    """
    修正xarray.Dataset的encoding
    """
    _ds.encoding['unlimited_dims'] = kwargs.get('unlimited_dims', 'time')
    _ds.encoding['format'] = kwargs.get('format', 'NETCDF4')
    _ds.encoding['engine'] = kwargs.get('engine', 'h5netcdf')
    _ds.encoding['dtype'] = kwargs.get('dtype', 'float32')
    _ds.encoding['complevel'] = kwargs.get('complevel', 4)
    _ds.encoding['zlib'] = kwargs.get('zlib', True)
    _ds.encoding['shuffle'] = kwargs.get('shuffle', True)
    _ds.encoding['fletcher32'] = kwargs.get('fletcher32', True)
    _ds.encoding['contiguous'] = kwargs.get('contiguous', True)
    _ds.encoding.update(kwargs)
    return _ds



#!/Users/christmas/opt/anaconda3/bin/python3
# -*- coding: utf-8 -*-
#  日期 : 2023/11/27 16:10
#  作者 : Christmas
#  邮箱 : 273519355@qq.com
#  项目 : Project
#  版本 : python 3
#  摘要 :
"""

"""
from .calculatefuns import *
from .CoorTrans import *
